@@ -121,7 +121,7 @@ AbstractCommand::~AbstractCommand() {
 void AbstractCommand::useFasterRequest
 (const SharedHandle<Request>& fasterRequest)
 {
-  A2_LOG_INFO(fmt("CUID#%lld - Use faster Request hostname=%s, port=%u",
+  A2_LOG_INFO(fmt("CUID#%" PRId64 " - Use faster Request hostname=%s, port=%u",
                   getCuid(),
                   fasterRequest->getHost().c_str(),
                   fasterRequest->getPort()));
@@ -135,7 +135,7 @@ void AbstractCommand::useFasterRequest
 }
 
 bool AbstractCommand::execute() {
-  A2_LOG_DEBUG(fmt("CUID#%lld - socket: read:%d, write:%d, hup:%d, err:%d",
+  A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - socket: read:%d, write:%d, hup:%d, err:%d",
                    getCuid(),
                    readEventEnabled(),
                    writeEventEnabled(),
@@ -146,7 +146,7 @@ bool AbstractCommand::execute() {
       return true;
     }
     if(req_ && req_->removalRequested()) {
-      A2_LOG_DEBUG(fmt("CUID#%lld - Discard original URI=%s because it is"
+      A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Discard original URI=%s because it is"
                        " requested.",
                        getCuid(), req_->getUri().c_str()));
       return prepareForRetry(0);
@@ -158,7 +158,7 @@ bool AbstractCommand::execute() {
         // This command previously has assigned segments, but it is
         // canceled. So discard current request chain.  Plus, if no
         // segment is available when http pipelining is used.
-        A2_LOG_DEBUG(fmt("CUID#%lld - It seems previously assigned segments"
+        A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - It seems previously assigned segments"
                          " are canceled. Restart.",
                          getCuid()));
         // Request::isPipeliningEnabled() == true means aria2
@@ -273,7 +273,7 @@ bool AbstractCommand::execute() {
         // empty.
         if(!req_->getConnectedAddr().empty()) {
           // Purging IP address cache to renew IP address.
-          A2_LOG_DEBUG(fmt("CUID#%lld - Marking IP address %s as bad",
+          A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Marking IP address %s as bad",
                            getCuid(),
                            req_->getConnectedAddr().c_str()));
           e_->markBadIPAddress(req_->getConnectedHostname(),
@@ -282,7 +282,7 @@ bool AbstractCommand::execute() {
         }
         if(e_->findCachedIPAddress
            (req_->getConnectedHostname(), req_->getConnectedPort()).empty()) {
-          A2_LOG_DEBUG(fmt("CUID#%lld - All IP addresses were marked bad."
+          A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - All IP addresses were marked bad."
                            " Removing Entry.",
                            getCuid()));
           e_->removeCachedIPAddress
@@ -320,7 +320,7 @@ bool AbstractCommand::execute() {
     req_->addTryCount();
     req_->resetRedirectCount();
     req_->resetUri();
-    const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
+    const int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
     bool isAbort = maxTries != 0 && req_->getTryCount() >= maxTries;
     if(isAbort) {
       A2_LOG_INFO(fmt(MSG_MAX_TRY,
@@ -370,14 +370,14 @@ void AbstractCommand::tryReserved() {
     // and there are no URI left. Because file length is unknown, we
     // can assume that there are no in-flight request object.
     if(entry->getLength() == 0 && entry->getRemainingUris().empty()) {
-      A2_LOG_DEBUG(fmt("CUID#%lld - Not trying next request."
+      A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Not trying next request."
                        " No reserved/pooled request is remaining and"
                        " total length is still unknown.",
                        getCuid()));
       return;
     }
   }
-  A2_LOG_DEBUG(fmt("CUID#%lld - Trying reserved/pooled request.",
+  A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Trying reserved/pooled request.",
                    getCuid()));
   std::vector<Command*> commands;
   requestGroup_->createNextCommand(commands, e_, 1);
@@ -397,7 +397,7 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
     req_->setMaxPipelinedRequest(1);
 
     fileEntry_->poolRequest(req_);
-    A2_LOG_DEBUG(fmt("CUID#%lld - Pooling request URI=%s",
+    A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Pooling request URI=%s",
                      getCuid(), req_->getUri().c_str()));
     if(getSegmentMan()) {
       getSegmentMan()->recognizeSegmentFor(fileEntry_);
@@ -422,7 +422,7 @@ void AbstractCommand::onAbort() {
     fileEntry_->removeIdenticalURI(req_->getUri());
     fileEntry_->removeRequest(req_);
   }
-  A2_LOG_DEBUG(fmt("CUID#%lld - Aborting download",
+  A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Aborting download",
                    getCuid()));
   if(getPieceStorage()) {
     getSegmentMan()->cancelSegment(getCuid());
@@ -440,10 +440,10 @@ void AbstractCommand::onAbort() {
         // Local file exists, but given servers(or at least contacted
         // ones) doesn't support resume. Let's restart download from
         // scratch.
-        A2_LOG_NOTICE(fmt("CUID#%lld - Failed to resume download."
-                          " Download from scratch.",
+        A2_LOG_NOTICE(fmt(_("CUID#%" PRId64 " - Failed to resume download."
+                            " Download from scratch."),
                           getCuid()));
-        A2_LOG_DEBUG(fmt("CUID#%lld - Gathering URIs that has CANNOT_RESUME"
+        A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Gathering URIs that has CANNOT_RESUME"
                          " error",
                          getCuid()));
         // Set PREF_ALWAYS_RESUME to A2_V_TRUE to avoid repeating this
@@ -459,7 +459,7 @@ void AbstractCommand::onAbort() {
           uris.reserve(res.size());
           std::transform(res.begin(), res.end(), std::back_inserter(uris),
                          std::mem_fun_ref(&URIResult::getURI));
-          A2_LOG_DEBUG(fmt("CUID#%lld - %lu URIs found.",
+          A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - %lu URIs found.",
                            getCuid(),
                            static_cast<unsigned long int>(uris.size())));
           fileEntry_->addUris(uris.begin(), uris.end());
@@ -543,35 +543,72 @@ void AbstractCommand::setWriteCheckSocketIf
 }
 
 namespace {
-// Returns proxy option value for the given protocol.
-const std::string& getProxyOptionFor
-(const std::string& proxyPref, const SharedHandle<Option>& option)
+// Constructs proxy URI, merging username and password if they are
+// defined.
+std::string makeProxyUri
+(const Pref* proxyPref,
+ const Pref* proxyUser,
+ const Pref* proxyPasswd,
+ const Option* option)
 {
-  if(option->defined(proxyPref)) {
-    return option->get(proxyPref);
+  uri::UriStruct us;
+  if(uri::parse(us, option->get(proxyPref))) {
+    if(option->defined(proxyUser)) {
+      us.username = option->get(proxyUser);
+    }
+    if(option->defined(proxyPasswd)) {
+      us.password = option->get(proxyPasswd);
+      us.hasPassword = true;
+    }
+    return uri::construct(us);
   } else {
-    return option->get(PREF_ALL_PROXY);
+    return "";
   }
 }
 } // namespace
 
 namespace {
+// Returns proxy option value for the given protocol.
+std::string getProxyOptionFor
+(const Pref* proxyPref,
+ const Pref* proxyUser,
+ const Pref* proxyPasswd,
+ const Option* option)
+{
+  std::string uri = makeProxyUri(proxyPref, proxyUser, proxyPasswd, option);
+  if(uri.empty()) {
+    return makeProxyUri(PREF_ALL_PROXY, PREF_ALL_PROXY_USER,
+                        PREF_ALL_PROXY_PASSWD, option);
+  } else {
+    return uri;
+  }
+}
+} // namespace
+
 // Returns proxy URI for given protocol.  If no proxy URI is defined,
 // then returns an empty string.
-const std::string& getProxyUri
-(const std::string& protocol, const SharedHandle<Option>& option)
+std::string getProxyUri
+(const std::string& protocol, const Option* option)
 {
   if(protocol == Request::PROTO_HTTP) {
-    return getProxyOptionFor(PREF_HTTP_PROXY, option);
+    return getProxyOptionFor(PREF_HTTP_PROXY,
+                             PREF_HTTP_PROXY_USER,
+                             PREF_HTTP_PROXY_PASSWD,
+                             option);
   } else if(protocol == Request::PROTO_HTTPS) {
-    return getProxyOptionFor(PREF_HTTPS_PROXY, option);
+    return getProxyOptionFor(PREF_HTTPS_PROXY,
+                             PREF_HTTPS_PROXY_USER,
+                             PREF_HTTPS_PROXY_PASSWD,
+                             option);
   } else if(protocol == Request::PROTO_FTP) {
-    return getProxyOptionFor(PREF_FTP_PROXY, option);
+    return getProxyOptionFor(PREF_FTP_PROXY,
+                             PREF_FTP_PROXY_USER,
+                             PREF_FTP_PROXY_PASSWD,
+                             option);
   } else {
     return A2STR::NIL;
   }
 }
-} // namespace
 
 namespace {
 // Returns true if proxy is defined for the given protocol. Otherwise
@@ -579,62 +616,38 @@ namespace {
 bool isProxyRequest
 (const std::string& protocol, const SharedHandle<Option>& option)
 {
-  const std::string& proxyUri = getProxyUri(protocol, option);
-  uri::UriStruct us;
-  return !proxyUri.empty() && uri::parse(us, proxyUri);
+  std::string proxyUri = getProxyUri(protocol, option.get());
+  return !proxyUri.empty();
 }
 } // namespace
 
-namespace {
-class DomainMatch {
-private:
-  std::string hostname_;
-public:
-  DomainMatch(const std::string& hostname):hostname_(hostname) {}
-
-  bool operator()(const std::string& domain) const
-  {
-    if(util::startsWith(domain, A2STR::DOT_C)) {
-      return util::endsWith(hostname_, domain);
-    } else {
-      return util::endsWith(hostname_, A2STR::DOT_C+domain);
-    }
-  }
-};
-} // namespace
-
 namespace {
 bool inNoProxy(const SharedHandle<Request>& req,
                const std::string& noProxy)
 {
-  std::vector<std::string> entries;
-  util::split(noProxy, std::back_inserter(entries), ",", true);
+  std::vector<Scip> entries;
+  util::splitIter(noProxy.begin(), noProxy.end(), std::back_inserter(entries),
+                  ',', true);
   if(entries.empty()) {
     return false;
   }
-  DomainMatch domainMatch(A2STR::DOT_C+req->getHost());
-  for(std::vector<std::string>::const_iterator i = entries.begin(),
+  for(std::vector<Scip>::const_iterator i = entries.begin(),
         eoi = entries.end(); i != eoi; ++i) {
-    std::string::size_type slashpos = (*i).find('/');
-    if(slashpos == std::string::npos) {
-      if(util::isNumericHost(*i)) {
-        if(req->getHost() == *i) {
-          return true;
-        }
-      } else if(domainMatch(*i)) {
+    std::string::const_iterator slashpos =
+      std::find((*i).first, (*i).second, '/');
+    if(slashpos == (*i).second) {
+      if(util::noProxyDomainMatch
+         (req->getHost(), std::string((*i).first, (*i).second))) {
         return true;
       }
     } else {
-      if(!util::isNumericHost(req->getHost())) {
-        // TODO We don't resolve hostname here.  More complete
-        // implementation is that we should first resolve
-        // hostname(which may result in several IP addresses) and
-        // evaluates against all of them
-        continue;
-      }
-      std::string ip = (*i).substr(0, slashpos);
+      // TODO We don't resolve hostname here.  More complete
+      // implementation is that we should first resolve
+      // hostname(which may result in several IP addresses) and
+      // evaluates against all of them
+      std::string ip((*i).first, slashpos);
       uint32_t bits;
-      if(!util::parseUIntNoThrow(bits, (*i).substr(slashpos+1))) {
+      if(!util::parseUIntNoThrow(bits, std::string(slashpos+1, (*i).second))) {
         continue;
       }
       if(util::inSameCidrBlock(ip, req->getHost(), bits)) {
@@ -658,13 +671,13 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
   if(inNoProxy(req_, getOption()->get(PREF_NO_PROXY))) {
     return proxyRequest;
   }
-  std::string proxy = getProxyUri(req_->getProtocol(), getOption());
+  std::string proxy = getProxyUri(req_->getProtocol(), getOption().get());
   if(!proxy.empty()) {
     proxyRequest.reset(new Request());
     if(proxyRequest->setUri(proxy)) {
-      A2_LOG_DEBUG(fmt("CUID#%lld - Using proxy", getCuid()));
+      A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Using proxy", getCuid()));
     } else {
-      A2_LOG_DEBUG(fmt("CUID#%lld - Failed to parse proxy string",
+      A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - Failed to parse proxy string",
                        getCuid()));
       proxyRequest.reset();
     }
@@ -869,7 +882,7 @@ void AbstractCommand::createSocket()
   socket_.reset(new SocketCore());
 }
 
-size_t AbstractCommand::calculateMinSplitSize() const
+int32_t AbstractCommand::calculateMinSplitSize() const
 {
   if(req_ && req_->isPipeliningEnabled()) {
     return getDownloadContext()->getPieceLength();
@@ -883,6 +896,11 @@ void AbstractCommand::setRequest(const SharedHandle<Request>& request)
   req_ = request;
 }
 
+void AbstractCommand::resetRequest()
+{
+  req_.reset();
+}
+
 void AbstractCommand::setFileEntry(const SharedHandle<FileEntry>& fileEntry)
 {
   fileEntry_ = fileEntry;
