@@ -74,6 +74,17 @@ void copyValues(const SharedHandle<FileEntry>& d,
 }
 } // namespace
 
+namespace {
+struct EntryCmp {
+  bool operator()
+  (const SharedHandle<FileEntry>& lhs,
+   const SharedHandle<FileEntry>& rhs) const
+  {
+    return lhs->getOriginalName() < rhs->getOriginalName();
+  }
+};
+} // namespace
+
 bool BtDependency::resolve()
 {
   if(!dependee_) {
@@ -115,47 +126,49 @@ bool BtDependency::resolve()
          dependantFileEntries[0]->getOriginalName().empty()) {
         copyValues(fileEntries[0], dependantFileEntries[0]);
       } else {
-        std::for_each(fileEntries.begin(), fileEntries.end(),
-                      std::bind2nd(mem_fun_sh(&FileEntry::setRequested),false));
+        std::vector<SharedHandle<FileEntry> > destFiles;
+        destFiles.reserve(fileEntries.size());
+        for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
+              fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
+          (*i)->setRequested(false);
+          destFiles.push_back(*i);
+        }
+        std::sort(destFiles.begin(), destFiles.end(), EntryCmp());
         // Copy file path in dependant_'s FileEntries to newly created
         // context's FileEntries to endorse the path structure of
         // dependant_.  URIs and singleHostMultiConnection are also copied.
-        std::vector<SharedHandle<FileEntry> >::const_iterator ctxFilesEnd =
-          fileEntries.end();
         for(std::vector<SharedHandle<FileEntry> >::const_iterator s =
               dependantFileEntries.begin(), eoi = dependantFileEntries.end();
             s != eoi; ++s){
           std::vector<SharedHandle<FileEntry> >::const_iterator d =
-            fileEntries.begin();
-          for(; d != ctxFilesEnd; ++d) {
-            if((*d)->getOriginalName() == (*s)->getOriginalName()) {
-              break;
-            }
-          }
-          if(d == ctxFilesEnd) {
+            std::lower_bound(destFiles.begin(), destFiles.end(), *s,
+                             EntryCmp());
+          if(d == destFiles.end() ||
+             (*d)->getOriginalName() != (*s)->getOriginalName()) {
             throw DL_ABORT_EX
               (fmt("No entry %s in torrent file",
                    (*s)->getOriginalName().c_str()));
+          } else {
+            copyValues(*d, *s);
           }
-          copyValues(*d, *s);
         }
       }
     } catch(RecoverableException& e) {
       A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
-      A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
-                      util::itos(dependant_->getGID()).c_str()));
+      A2_LOG_INFO(fmt("BtDependency for GID#%" PRId64 " failed. Go without Bt.",
+                      dependant_->getGID()));
       return true;
     }
-    A2_LOG_INFO(fmt("Dependency resolved for GID#%s",
-                    util::itos(dependant_->getGID()).c_str()));
+    A2_LOG_INFO(fmt("Dependency resolved for GID#%" PRId64 "",
+                    dependant_->getGID()));
     dependant_->setDownloadContext(context);
     return true;
   } else if(dependee_->getNumCommand() == 0) {
     // dependee_'s download failed.
     // cut reference here
     dependee_.reset();
-    A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
-                    util::itos(dependant_->getGID()).c_str()));
+    A2_LOG_INFO(fmt("BtDependency for GID#%" PRId64 " failed. Go without Bt.",
+                    dependant_->getGID()));
     return true;
   } else {
     return false;
