@@ -55,33 +55,29 @@
 #include "PeerConnection.h"
 #include "fmt.h"
 #include "DownloadContext.h"
+#include "PeerStorage.h"
 
 namespace aria2 {
 
 const std::string BtPieceMessage::NAME("piece");
 
 BtPieceMessage::BtPieceMessage
-(size_t index, uint32_t begin, size_t blockLength)
+(size_t index, int32_t begin, int32_t blockLength)
   : AbstractBtMessage(ID, NAME),
     index_(index),
     begin_(begin),
     blockLength_(blockLength),
-    block_(0),
-    rawData_(0)
+    data_(0)
 {
   setUploading(true);
 }
 
 BtPieceMessage::~BtPieceMessage()
-{
-  delete [] rawData_;
-}
+{}
 
-void BtPieceMessage::setRawMessage(unsigned char* data)
+void BtPieceMessage::setMsgPayload(const unsigned char* data)
 {
-  delete [] rawData_;
-  rawData_ = data;
-  block_ = data+9;
+  data_ = data;
 }
 
 BtPieceMessageHandle BtPieceMessage::create
@@ -107,36 +103,38 @@ void BtPieceMessage::doReceivedAction()
   if(!RequestSlot::isNull(slot)) {
     getPeer()->snubbing(false);
     SharedHandle<Piece> piece = getPieceStorage()->getPiece(index_);
-    off_t offset = (off_t)index_*downloadContext_->getPieceLength()+begin_;
+    int64_t offset =
+      static_cast<int64_t>(index_)*downloadContext_->getPieceLength()+begin_;
     A2_LOG_DEBUG(fmt(MSG_PIECE_RECEIVED,
                      getCuid(),
                      static_cast<unsigned long>(index_),
                      begin_,
                      blockLength_,
-                     static_cast<long long int>(offset),
+                     static_cast<int64_t>(offset),
                      static_cast<unsigned long>(slot.getBlockIndex())));
     if(piece->hasBlock(slot.getBlockIndex())) {
       A2_LOG_DEBUG("Already have this block.");
       return;
     }
     getPieceStorage()->getDiskAdaptor()->writeData
-      (block_, blockLength_, offset);
+      (data_+9, blockLength_, offset);
     piece->completeBlock(slot.getBlockIndex());
     A2_LOG_DEBUG(fmt(MSG_PIECE_BITFIELD, getCuid(),
                      util::toHex(piece->getBitfield(),
                                  piece->getBitfieldLength()).c_str()));
-    piece->updateHash(begin_, block_, blockLength_);
+    piece->updateHash(begin_, data_+9, blockLength_);
     getBtMessageDispatcher()->removeOutstandingRequest(slot);
     if(piece->pieceComplete()) {
       if(checkPieceHash(piece)) {
         onNewPiece(piece);
       } else {
         onWrongPiece(piece);
+        peerStorage_->addBadPeer(getPeer()->getIPAddress());
         throw DL_ABORT_EX("Bad piece hash.");
       }
     }
   } else {
-    A2_LOG_DEBUG(fmt("CUID#%lld - RequestSlot not found, index=%lu, begin=%u",
+    A2_LOG_DEBUG(fmt("CUID#%" PRId64 " - RequestSlot not found, index=%lu, begin=%d",
                      getCuid(),
                      static_cast<unsigned long>(index_),
                      begin_));
@@ -184,8 +182,8 @@ void BtPieceMessage::send()
     A2_LOG_DEBUG(fmt("msglength = %lu bytes",
                      static_cast<unsigned long>(msgHdrLen+blockLength_)));
     getPeerConnection()->pushBytes(msgHdr, msgHdrLen);
-    off_t pieceDataOffset =
-      (off_t)index_*downloadContext_->getPieceLength()+begin_;
+    int64_t pieceDataOffset =
+      static_cast<int64_t>(index_)*downloadContext_->getPieceLength()+begin_;
     pushPieceData(pieceDataOffset, blockLength_);
   }
   writtenLength = getPeerConnection()->sendPendingData();
@@ -193,7 +191,7 @@ void BtPieceMessage::send()
   setSendingInProgress(!getPeerConnection()->sendBufferIsEmpty());
 }
 
-void BtPieceMessage::pushPieceData(off_t offset, size_t length) const
+void BtPieceMessage::pushPieceData(int64_t offset, int32_t length) const
 {
   assert(length <= 16*1024);
   unsigned char* buf = new unsigned char[length];
@@ -204,7 +202,7 @@ void BtPieceMessage::pushPieceData(off_t offset, size_t length) const
     delete [] buf;
     throw;
   }
-  if(r == static_cast<ssize_t>(length)) {
+  if(r == length) {
     getPeerConnection()->pushBytes(buf, length);
   } else {
     throw DL_ABORT_EX(EX_DATA_READ);
@@ -213,8 +211,10 @@ void BtPieceMessage::pushPieceData(off_t offset, size_t length) const
 
 std::string BtPieceMessage::toString() const
 {
-  return strconcat(NAME, " index=", util::itos(index_), ", begin=",
-                   util::itos(begin_), ", length=", util::itos(blockLength_));
+  return fmt("%s index=%lu, begin=%d, length=%d",
+             NAME.c_str(),
+             static_cast<unsigned long>(index_),
+             begin_, blockLength_);
 }
 
 bool BtPieceMessage::checkPieceHash(const SharedHandle<Piece>& piece)
@@ -223,11 +223,11 @@ bool BtPieceMessage::checkPieceHash(const SharedHandle<Piece>& piece)
     A2_LOG_DEBUG(fmt("Hash is available!! index=%lu",
                      static_cast<unsigned long>(piece->getIndex())));
     return
-      piece->getHashString()==downloadContext_->getPieceHash(piece->getIndex());
+      piece->getDigest() == downloadContext_->getPieceHash(piece->getIndex());
   } else {
-    off_t offset = (off_t)piece->getIndex()*downloadContext_->getPieceLength();
-    
-    return message_digest::staticSHA1DigestHexDigest
+    int64_t offset = static_cast<int64_t>(piece->getIndex())
+      *downloadContext_->getPieceLength();
+    return message_digest::staticSHA1Digest
       (getPieceStorage()->getDiskAdaptor(), offset, piece->getLength())
       == downloadContext_->getPieceHash(piece->getIndex());
   }
@@ -258,7 +258,8 @@ void BtPieceMessage::erasePieceOnDisk(const SharedHandle<Piece>& piece)
   size_t BUFSIZE = 4096;
   unsigned char buf[BUFSIZE];
   memset(buf, 0, BUFSIZE);
-  off_t offset = (off_t)piece->getIndex()*downloadContext_->getPieceLength();
+  int64_t offset =
+    static_cast<int64_t>(piece->getIndex())*downloadContext_->getPieceLength();
   div_t res = div(piece->getLength(), BUFSIZE);
   for(int i = 0; i < res.quot; ++i) {
     getPieceStorage()->getDiskAdaptor()->writeData(buf, BUFSIZE, offset);
@@ -318,4 +319,10 @@ void BtPieceMessage::setDownloadContext
   downloadContext_ = downloadContext;
 }
 
+void BtPieceMessage::setPeerStorage
+(const SharedHandle<PeerStorage>& peerStorage)
+{
+  peerStorage_ = peerStorage;
+}
+
 } // namespace aria2
