@@ -42,29 +42,26 @@ namespace aria2 {
 
 class BtRuntime {
 private:
-  uint64_t uploadLengthAtStartup_;
+  int64_t uploadLengthAtStartup_;
   bool halt_;
-  unsigned int connections_;
+  int connections_;
   bool ready_;
   // Maximum number of peers to hold connections at the same time.
   // 0 means unlimited.
-  unsigned int maxPeers_;
+  int maxPeers_;
   // Minimum number of peers. This value is used for getting more peers from
   // tracker. 0 means always the number of peers is under minimum.
-  unsigned int minPeers_;
-
-  static const unsigned int DEFAULT_MIN_PEERS = 40;
-
+  int minPeers_;
 public:
   BtRuntime();
 
   ~BtRuntime();
 
-  uint64_t getUploadLengthAtStartup() const {
+  int64_t getUploadLengthAtStartup() const {
     return uploadLengthAtStartup_;
   }
 
-  void setUploadLengthAtStartup(uint64_t length) {
+  void setUploadLengthAtStartup(int64_t length) {
     uploadLengthAtStartup_ = length;
   }
 
@@ -74,7 +71,7 @@ public:
     halt_ = halt;
   }
 
-  unsigned int getConnections() const { return connections_; }
+  int getConnections() const { return connections_; }
 
   void increaseConnections() { ++connections_; }
 
@@ -99,14 +96,15 @@ public:
 
   void setReady(bool go) { ready_ = go; }
 
-  void setMaxPeers(unsigned int maxPeers);
+  void setMaxPeers(int maxPeers);
 
-  unsigned int getMaxPeers() const
+  int getMaxPeers() const
   {
     return maxPeers_;
   }
 
-  static const unsigned int DEFAULT_MAX_PEERS = 55;
+  static const int DEFAULT_MAX_PEERS = 55;
+  static const int DEFAULT_MIN_PEERS = 40;
 };
 
 typedef SharedHandle<BtRuntime> BtRuntimeHandle;
