@@ -37,16 +37,17 @@
 #include "CheckIntegrityCommand.h"
 #include "message.h"
 #include "Logger.h"
-#include "FileEntry.h"
+#include "LogFactory.h"
 #include "util.h"
+#include "fmt.h"
 
 namespace aria2 {
 
 CheckIntegrityDispatcherCommand::CheckIntegrityDispatcherCommand
 (cuid_t cuid,
  const SharedHandle<CheckIntegrityMan>& fileAllocMan,
- DownloadEngine* e):SequentialDispatcherCommand<CheckIntegrityEntry>
-                    (cuid, fileAllocMan, e)
+ DownloadEngine* e)
+  : SequentialDispatcherCommand<CheckIntegrityEntry>(cuid, fileAllocMan, e)
 {
   setStatusRealtime();
 }
@@ -54,13 +55,11 @@ CheckIntegrityDispatcherCommand::CheckIntegrityDispatcherCommand
 Command* CheckIntegrityDispatcherCommand::createCommand
 (const SharedHandle<CheckIntegrityEntry>& entry)
 {
-  cuid_t newCUID = _e->newCUID();
-  if(logger->info()) {
-    logger->info("CUID#%s - Dispatching CheckIntegrityCommand CUID#%s.",
-                 util::itos(cuid).c_str(), util::itos(newCUID).c_str());
-  }
+  cuid_t newCUID = getDownloadEngine()->newCUID();
+  A2_LOG_INFO(fmt("CUID#%" PRId64 " - Dispatching CheckIntegrityCommand CUID#%" PRId64 ".",
+                  getCuid(), newCUID));
   return new CheckIntegrityCommand
-    (newCUID, entry->getRequestGroup(), _e, entry);
+    (newCUID, entry->getRequestGroup(), getDownloadEngine(), entry);
 }
 
 } // namespace aria2
