@@ -56,8 +56,8 @@ DownloadContext::DownloadContext():
   downloadStopTime_(downloadStartTime_),
   metalinkServerContacted_(false) {}
 
-DownloadContext::DownloadContext(size_t pieceLength,
-                                 uint64_t totalLength,
+DownloadContext::DownloadContext(int32_t pieceLength,
+                                 int64_t totalLength,
                                  const std::string& path):
   pieceLength_(pieceLength),
   checksumVerified_(false),
@@ -95,12 +95,10 @@ int64_t DownloadContext::calculateSessionTime() const
 }
 
 SharedHandle<FileEntry>
-DownloadContext::findFileEntryByOffset(off_t offset) const
+DownloadContext::findFileEntryByOffset(int64_t offset) const
 {
   if(fileEntries_.empty() ||
-     (offset > 0 &&
-      fileEntries_.back()->getOffset()+fileEntries_.back()->getLength() <=
-      static_cast<uint64_t>(offset))){
+     (offset > 0 && fileEntries_.back()->getLastOffset() <= offset)) {
     return SharedHandle<FileEntry>();
   }
 
@@ -128,22 +126,27 @@ void DownloadContext::setFilePathWithIndex
   }
 }
 
-void DownloadContext::setFileFilter(IntSequence seq)
+void DownloadContext::setFileFilter(SegList<int>& sgl)
 {
-  std::vector<int32_t> fileIndexes = seq.flush();
-  std::sort(fileIndexes.begin(), fileIndexes.end());
-  fileIndexes.erase(std::unique(fileIndexes.begin(), fileIndexes.end()),
-                    fileIndexes.end());
-
-  bool selectAll = fileIndexes.empty() || fileEntries_.size() == 1;
-    
-  int32_t index = 1;
-  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries_.begin(), eoi = fileEntries_.end();
-      i != eoi; ++i, ++index) {
-    (*i)->setRequested
-      (selectAll ||
-       std::binary_search(fileIndexes.begin(), fileIndexes.end(), index));
+  if(!sgl.hasNext() || fileEntries_.size() == 1) {
+    std::for_each(fileEntries_.begin(), fileEntries_.end(),
+                  std::bind2nd(mem_fun_sh(&FileEntry::setRequested), true));
+    return;
+  }
+  assert(sgl.peek() >= 1);
+  size_t len = fileEntries_.size();
+  size_t i = 0;
+  for(; i < len && sgl.hasNext(); ++i) {
+    size_t idx = sgl.peek()-1;
+    if(i == idx) {
+      fileEntries_[i]->setRequested(true);
+      sgl.next();
+    } else if(i < idx) {
+      fileEntries_[i]->setRequested(false);
+    }
+  }
+  for(; i < len; ++i) {
+    fileEntries_[i]->setRequested(false);
   }
 }
 
@@ -180,6 +183,7 @@ void DownloadContext::releaseRuntimeResource()
 {
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
         fileEntries_.begin(), eoi = fileEntries_.end(); i != eoi; ++i) {
+    (*i)->putBackRequest();
     (*i)->releaseRuntimeResource();
   }
 }
@@ -194,7 +198,7 @@ size_t DownloadContext::getNumPieces() const
   }
 }
 
-uint64_t DownloadContext::getTotalLength() const
+int64_t DownloadContext::getTotalLength() const
 {
   if(fileEntries_.empty()) {
     return 0;
