@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DOWNLOAD_RESULT_H_
-#define _D_DOWNLOAD_RESULT_H_
+#ifndef D_DOWNLOAD_RESULT_H
+#define D_DOWNLOAD_RESULT_H
 
 #include "common.h"
 
@@ -43,18 +43,18 @@
 #include <vector>
 
 #include "SharedHandle.h"
-#include "DownloadResultCode.h"
+#include "error_code.h"
 #include "RequestGroup.h"
-#include "Option.h"
-#include "MetadataInfo.h"
 
 namespace aria2 {
 
+class Option;
 class FileEntry;
+class MetadataInfo;
 
 struct DownloadResult
 {
-  gid_t gid;
+  a2_gid_t gid;
  
   std::vector<SharedHandle<FileEntry> > fileEntries;
 
@@ -65,39 +65,46 @@ struct DownloadResult
   // milliseconds
   int64_t sessionTime;
 
-  downloadresultcode::RESULT result;
+  error_code::Value result;
 
   // This field contains GIDs. See comment in
-  // RequestGroup.cc::_followedByGIDs.
-  std::vector<gid_t> followedBy;
+  // RequestGroup.cc::followedByGIDs_.
+  std::vector<a2_gid_t> followedBy;
 
   // This field contains GID. See comment in
-  // RequestGroup.cc::_belongsToGID.
-  gid_t belongsTo;
+  // RequestGroup.cc::belongsToGID_.
+  a2_gid_t belongsTo;
 
   SharedHandle<Option> option;
 
   SharedHandle<MetadataInfo> metadataInfo;
 
-  uint64_t totalLength;
+  int64_t totalLength;
 
-  uint64_t completedLength;
+  int64_t completedLength;
 
-  uint64_t uploadLength;
+  int64_t uploadLength;
 
-  std::string bitfieldStr;
+  std::string bitfield;
 
-  std::string infoHashStr;
+  std::string infoHash;
 
-  size_t pieceLength;
+  int32_t pieceLength;
 
   size_t numPieces;
 
   std::string dir;
+
+  DownloadResult();
+  ~DownloadResult();
+
+  // Don't allow copying
+  DownloadResult(const DownloadResult& c);
+  DownloadResult& operator=(const DownloadResult& c);
 };
 
 typedef SharedHandle<DownloadResult> DownloadResultHandle;
 
 } // namespace aria2
 
-#endif // _D_DOWNLOAD_RESULT_H_
+#endif // D_DOWNLOAD_RESULT_H
