@@ -37,12 +37,13 @@
 #include "Request.h"
 #include "Socket.h"
 #include "DownloadContext.h"
+#include "SocketRecvBuffer.h"
 
 namespace aria2 {
 
 FtpTunnelRequestCommand::FtpTunnelRequestCommand
-(int cuid,
- const RequestHandle& req,
+(cuid_t cuid,
+ const SharedHandle<Request>& req,
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  DownloadEngine* e,
@@ -58,7 +59,8 @@ FtpTunnelRequestCommand::~FtpTunnelRequestCommand() {}
 Command* FtpTunnelRequestCommand::getNextCommand()
 {
   return new FtpTunnelResponseCommand
-    (cuid, req, _fileEntry, _requestGroup, httpConnection, e, socket);
+    (getCuid(), getRequest(), getFileEntry(), getRequestGroup(),
+     getHttpConnection(), getDownloadEngine(), getSocket());
 }
 
 } // namespace aria2
