@@ -55,36 +55,45 @@
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
 #include "a2functional.h"
+#include "fmt.h"
+#include "SocketRecvBuffer.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
 HttpRequestEntry::HttpRequestEntry
-(const SharedHandle<HttpRequest>& httpRequest):
-  httpRequest_(httpRequest),
-  proc_(new HttpHeaderProcessor()) {}
+(const SharedHandle<HttpRequest>& httpRequest)
+  : httpRequest_(httpRequest),
+    proc_(new HttpHeaderProcessor(HttpHeaderProcessor::CLIENT_PARSER))
+{}
 
 HttpRequestEntry::~HttpRequestEntry() {}
 
-HttpConnection::HttpConnection(cuid_t cuid, const SocketHandle& socket):
-  cuid_(cuid), socket_(socket),
-  socketBuffer_(socket),
-  logger_(LogFactory::getInstance())
+HttpConnection::HttpConnection
+(cuid_t cuid,
+ const SocketHandle& socket,
+ const SharedHandle<SocketRecvBuffer>& socketRecvBuffer)
+  : cuid_(cuid),
+    socket_(socket),
+    socketRecvBuffer_(socketRecvBuffer),
+    socketBuffer_(socket)
 {}
 
+HttpConnection::~HttpConnection() {}
+
 std::string HttpConnection::eraseConfidentialInfo(const std::string& request)
 {
   std::istringstream istr(request);
   std::string result;
   std::string line;
   while(getline(istr, line)) {
-    static const std::string AUTH_HEADER("Authorization: Basic");
-    static const std::string PROXY_AUTH_HEADER("Proxy-Authorization: Basic");
-    if(util::startsWith(line, AUTH_HEADER)) {
+    if(util::startsWith(line, "Authorization: Basic")) {
       result += "Authorization: Basic ********\n";
-    } else if(util::startsWith(line, PROXY_AUTH_HEADER)) {
+    } else if(util::startsWith(line, "Proxy-Authorization: Basic")) {
       result += "Proxy-Authorization: Basic ********\n";
     } else {
-      strappend(result, line, "\n");
+      result += line;
+      result += "\n";
     }
   }
   return result;
@@ -93,11 +102,9 @@ std::string HttpConnection::eraseConfidentialInfo(const std::string& request)
 void HttpConnection::sendRequest(const SharedHandle<HttpRequest>& httpRequest)
 {
   std::string request = httpRequest->createRequest();
-  if(logger_->info()) {
-    logger_->info(MSG_SENDING_REQUEST,
-                  util::itos(cuid_).c_str(),
-                  eraseConfidentialInfo(request).c_str());
-  }
+  A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                  cuid_,
+                  eraseConfidentialInfo(request).c_str()));
   socketBuffer_.pushStr(request);
   socketBuffer_.send();
   SharedHandle<HttpRequestEntry> entry(new HttpRequestEntry(httpRequest));
@@ -108,11 +115,9 @@ void HttpConnection::sendProxyRequest
 (const SharedHandle<HttpRequest>& httpRequest)
 {
   std::string request = httpRequest->createProxyRequest();
-  if(logger_->info()) {
-    logger_->info(MSG_SENDING_REQUEST,
-                  util::itos(cuid_).c_str(),
-                  eraseConfidentialInfo(request).c_str());
-  }
+  A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                  cuid_,
+                  eraseConfidentialInfo(request).c_str()));
   socketBuffer_.pushStr(request);
   socketBuffer_.send();
   SharedHandle<HttpRequestEntry> entry(new HttpRequestEntry(httpRequest));
@@ -125,38 +130,28 @@ SharedHandle<HttpResponse> HttpConnection::receiveResponse()
     throw DL_ABORT_EX(EX_NO_HTTP_REQUEST_ENTRY_FOUND);
   }
   HttpRequestEntryHandle entry = outstandingHttpRequests_.front();
-  HttpHeaderProcessorHandle proc = entry->getHttpHeaderProcessor();
-
-  unsigned char buf[512];
-  size_t size = sizeof(buf);
-  socket_->peekData(buf, size);
-  if(size == 0) {
-    if(socket_->wantRead() || socket_->wantWrite()) {
-      return SharedHandle<HttpResponse>();
-    } else {
-      throw DL_RETRY_EX(EX_INVALID_RESPONSE);
+  const SharedHandle<HttpHeaderProcessor>& proc =
+    entry->getHttpHeaderProcessor();
+  if(socketRecvBuffer_->bufferEmpty()) {
+    if(socketRecvBuffer_->recv() == 0 &&
+       !socket_->wantRead() && !socket_->wantWrite()) {
+      throw DL_RETRY_EX(EX_GOT_EOF);
     }
   }
-  proc->update(buf, size);
-  if(!proc->eoh()) {
-    socket_->readData(buf, size);
-    return SharedHandle<HttpResponse>();
+  SharedHandle<HttpResponse> httpResponse;
+  if(proc->parse(socketRecvBuffer_->getBuffer(),
+                 socketRecvBuffer_->getBufferLength())) {
+    const SharedHandle<HttpHeader>& httpHeader = proc->getResult();
+    A2_LOG_INFO(fmt(MSG_RECEIVE_RESPONSE,
+                    cuid_,
+                    proc->getHeaderString().c_str()));
+    httpResponse.reset(new HttpResponse());
+    httpResponse->setCuid(cuid_);
+    httpResponse->setHttpHeader(httpHeader);
+    httpResponse->setHttpRequest(entry->getHttpRequest());
+    outstandingHttpRequests_.pop_front();
   }
-  size_t putbackDataLength = proc->getPutBackDataLength();
-  size -= putbackDataLength;
-  socket_->readData(buf, size);
-  if(logger_->info()) {
-    logger_->info(MSG_RECEIVE_RESPONSE,
-                  util::itos(cuid_).c_str(), proc->getHeaderString().c_str());
-  }
-  SharedHandle<HttpHeader> httpHeader = proc->getHttpResponseHeader();
-  SharedHandle<HttpResponse> httpResponse(new HttpResponse());
-  httpResponse->setCuid(cuid_);
-  httpResponse->setHttpHeader(httpHeader);
-  httpResponse->setHttpRequest(entry->getHttpRequest());
-
-  outstandingHttpRequests_.pop_front();
-
+  socketRecvBuffer_->shiftBuffer(proc->getLastBytesProcessed());
   return httpResponse;
 }
 
@@ -165,7 +160,7 @@ bool HttpConnection::isIssued(const SharedHandle<Segment>& segment) const
   for(HttpRequestEntries::const_iterator itr = outstandingHttpRequests_.begin(),
         eoi = outstandingHttpRequests_.end(); itr != eoi; ++itr) {
     SharedHandle<HttpRequest> httpRequest = (*itr)->getHttpRequest();
-    if(httpRequest->getSegment() == segment) {
+    if(*httpRequest->getSegment() == *segment) {
       return true;
     }
   }
