@@ -53,9 +53,6 @@
 
 namespace aria2 {
 
-#define BUFSIZE (256*1024)
-#define ALIGNMENT 512
-
 IteratableChunkChecksumValidator::IteratableChunkChecksumValidator
 (const SharedHandle<DownloadContext>& dctx,
  const PieceStorageHandle& pieceStorage)
@@ -63,14 +60,10 @@ IteratableChunkChecksumValidator::IteratableChunkChecksumValidator
     pieceStorage_(pieceStorage),
     bitfield_(new BitfieldMan(dctx_->getPieceLength(),
                               dctx_->getTotalLength())),
-    currentIndex_(0),
-    buffer_(0)
+    currentIndex_(0)
 {}
 
-IteratableChunkChecksumValidator::~IteratableChunkChecksumValidator()
-{
-  delete [] buffer_;
-}
+IteratableChunkChecksumValidator::~IteratableChunkChecksumValidator() {}
 
 
 void IteratableChunkChecksumValidator::validateChunk()
@@ -82,11 +75,12 @@ void IteratableChunkChecksumValidator::validateChunk()
       if(actualChecksum == dctx_->getPieceHashes()[currentIndex_]) {
         bitfield_->setBit(currentIndex_);
       } else {
-        A2_LOG_INFO(fmt(EX_INVALID_CHUNK_CHECKSUM,
-                        static_cast<unsigned long>(currentIndex_),
-                        util::itos(getCurrentOffset(), true).c_str(),
-                        dctx_->getPieceHashes()[currentIndex_].c_str(),
-                        actualChecksum.c_str()));
+        A2_LOG_INFO
+          (fmt(EX_INVALID_CHUNK_CHECKSUM,
+               static_cast<unsigned long>(currentIndex_),
+               static_cast<int64_t>(getCurrentOffset()),
+               util::toHex(dctx_->getPieceHashes()[currentIndex_]).c_str(),
+               util::toHex(actualChecksum).c_str()));
         bitfield_->unsetBit(currentIndex_);
       }
     } catch(RecoverableException& ex) {
@@ -107,7 +101,7 @@ void IteratableChunkChecksumValidator::validateChunk()
 
 std::string IteratableChunkChecksumValidator::calculateActualChecksum()
 {
-  off_t offset = getCurrentOffset();
+  int64_t offset = getCurrentOffset();
   size_t length;
   // When validating last piece
   if(currentIndex_+1 == dctx_->getNumPieces()) {
@@ -120,46 +114,30 @@ std::string IteratableChunkChecksumValidator::calculateActualChecksum()
 
 void IteratableChunkChecksumValidator::init()
 {
-  delete [] buffer_;
-  buffer_ = new unsigned char[BUFSIZE];
   ctx_ = MessageDigest::create(dctx_->getPieceHashType());
   bitfield_->clearAllBit();
   currentIndex_ = 0;
 }
 
-std::string IteratableChunkChecksumValidator::digest(off_t offset, size_t length)
+std::string IteratableChunkChecksumValidator::digest(int64_t offset, size_t length)
 {
+  unsigned char buf[4096];
   ctx_->reset();
-  off_t curoffset = offset/ALIGNMENT*ALIGNMENT;
-  off_t max = offset+length;
-  off_t woffset;
-  if(curoffset < offset) {
-    woffset = offset-curoffset;
-  } else {
-    woffset = 0;
-  }
-  while(curoffset < max) {
-    size_t r = pieceStorage_->getDiskAdaptor()->readData(buffer_, BUFSIZE,
-                                                         curoffset);
-    if(r == 0 || r < static_cast<size_t>(woffset)) {
+  int64_t max = offset+length;
+  while(offset < max) {
+    size_t r = pieceStorage_->getDiskAdaptor()->readData
+      (buf, std::min(static_cast<int64_t>(sizeof(buf)), max-offset), offset);
+    if(r == 0) {
       throw DL_ABORT_EX
         (fmt(EX_FILE_READ, dctx_->getBasePath().c_str(),
              "data is too short"));
     }
-    size_t wlength;
-    if(max < static_cast<off_t>(curoffset+r)) {
-      wlength = max-curoffset-woffset;
-    } else {
-      wlength = r-woffset;
-    }
-    ctx_->update(buffer_+woffset, wlength);
-    curoffset += r;
-    woffset = 0;
+    ctx_->update(buf, r);
+    offset += r;
   }
-  return ctx_->hexDigest();
+  return ctx_->digest();
 }
 
-
 bool IteratableChunkChecksumValidator::finished() const
 {
   if(currentIndex_ >= dctx_->getNumPieces()) {
@@ -169,12 +147,12 @@ bool IteratableChunkChecksumValidator::finished() const
   }
 }
 
-off_t IteratableChunkChecksumValidator::getCurrentOffset() const
+int64_t IteratableChunkChecksumValidator::getCurrentOffset() const
 {
-  return (off_t)currentIndex_*dctx_->getPieceLength();
+  return static_cast<int64_t>(currentIndex_)*dctx_->getPieceLength();
 }
 
-uint64_t IteratableChunkChecksumValidator::getTotalLength() const
+int64_t IteratableChunkChecksumValidator::getTotalLength() const
 {
   return dctx_->getTotalLength();
 }
