@@ -54,11 +54,10 @@ private:
   SharedHandle<BitfieldMan> bitfield_;
   size_t currentIndex_;
   SharedHandle<MessageDigest> ctx_;
-  unsigned char* buffer_;
 
   std::string calculateActualChecksum();
 
-  std::string digest(off_t offset, size_t length);
+  std::string digest(int64_t offset, size_t length);
 
 public:
   IteratableChunkChecksumValidator(const SharedHandle<DownloadContext>& dctx,
@@ -72,9 +71,9 @@ public:
 
   virtual bool finished() const;
 
-  virtual off_t getCurrentOffset() const;
+  virtual int64_t getCurrentOffset() const;
 
-  virtual uint64_t getTotalLength() const;
+  virtual int64_t getTotalLength() const;
 };
 
 typedef SharedHandle<IteratableChunkChecksumValidator> IteratableChunkChecksumValidatorHandle;
