@@ -110,6 +110,18 @@ MetalinkParserStateMachine::MetalinkParserStateMachine():
   stateStack_.push(initialState_);
 }
 
+MetalinkParserStateMachine::~MetalinkParserStateMachine() {}
+
+void MetalinkParserStateMachine::reset()
+{
+  ctrl_->reset();
+  errors_.clear();
+  while(!stateStack_.empty()) {
+    stateStack_.pop();
+  }
+  stateStack_.push(initialState_);
+}
+
 void MetalinkParserStateMachine::setMetalinkState()
 {
   stateStack_.push(metalinkState_);
@@ -260,7 +272,7 @@ void MetalinkParserStateMachine::setFileNameOfEntry(const std::string& filename)
   ctrl_->setFileNameOfEntry(filename);
 }
 
-void MetalinkParserStateMachine::setFileLengthOfEntry(uint64_t length)
+void MetalinkParserStateMachine::setFileLengthOfEntry(int64_t length)
 {
   ctrl_->setFileLengthOfEntry(length);
 }
@@ -500,18 +512,18 @@ void MetalinkParserStateMachine::cancelMetaurlTransaction()
 }
 
 void MetalinkParserStateMachine::beginElement
-(const std::string& localname,
- const std::string& prefix,
- const std::string& nsUri,
+(const char* localname,
+ const char* prefix,
+ const char* nsUri,
  const std::vector<XmlAttr>& attrs)
 {
   stateStack_.top()->beginElement(this, localname, prefix, nsUri, attrs);
 }
   
 void MetalinkParserStateMachine::endElement
-(const std::string& localname,
- const std::string& prefix,
- const std::string& nsUri,
+(const char* localname,
+ const char* prefix,
+ const char* nsUri,
  const std::string& characters)
 {
   stateStack_.top()->endElement(this, localname, prefix, nsUri, characters);
@@ -539,5 +551,9 @@ std::string MetalinkParserStateMachine::getErrorString() const
   return error.str();
 }
 
+void MetalinkParserStateMachine::setBaseUri(const std::string& uri)
+{
+  ctrl_->setBaseUri(uri);
+}
 
 } // namespace aria2
