@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -32,13 +32,16 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_PEER_STORAGE_H_
-#define _D_PEER_STORAGE_H_
+#ifndef D_PEER_STORAGE_H
+#define D_PEER_STORAGE_H
 
 #include "common.h"
+
+#include <deque>
+#include <vector>
+
 #include "SharedHandle.h"
 #include "TransferStat.h"
-#include <deque>
 
 namespace aria2 {
 
@@ -57,13 +60,24 @@ public:
   /**
    * Adds all peers in peers to internal peer list.
    */
-  virtual void addPeer(const std::deque<SharedHandle<Peer> >& peers) = 0;
+  virtual void addPeer(const std::vector<SharedHandle<Peer> >& peers) = 0;
 
   /**
    * Returns internal peer list.
    */
   virtual const std::deque<SharedHandle<Peer> >& getPeers() = 0;
 
+
+  /**
+   * Returns the number of peers.
+   */
+  virtual size_t countPeer() const = 0;
+
+  /**
+   * Returns internal dropped peer list.
+   */
+  virtual const std::deque<SharedHandle<Peer> >& getDroppedPeers() = 0;
+
   /**
    * Returns one of the unused peers.
    */
@@ -78,7 +92,7 @@ public:
   /**
    * Returns the list of peers which are currently connected from localhost.
    */
-  virtual void getActivePeers(std::deque<SharedHandle<Peer> >& peers) = 0;
+  virtual void getActivePeers(std::vector<SharedHandle<Peer> >& peers) = 0;
 
   /**
    * Calculates current download/upload statistics.
@@ -89,6 +103,17 @@ public:
 
   virtual TransferStat getTransferStatFor(const SharedHandle<Peer>& peer) = 0;
 
+  /**
+   * Returns true if peer with ipaddr should be ignored because, for
+   * example, it sends bad data.
+   */
+  virtual bool isBadPeer(const std::string& ipaddr) = 0;
+
+  /**
+   * Adds peer with ipaddr in bad peer set.
+   */
+  virtual void addBadPeer(const std::string& ipaddr) = 0;
+
   /**
    * Tells PeerStorage object that peer is no longer used in the session.
    */
@@ -103,4 +128,4 @@ typedef SharedHandle<PeerStorage> PeerStorageHandle;
 
 } // namespace aria2
 
-#endif // _D_PEER_STORAGE_H_
+#endif // D_PEER_STORAGE_H
