@@ -73,7 +73,7 @@ class BtRuntime;
 class PeerStorage;
 #endif // ENABLE_BITTORRENT
 
-typedef int64_t gid_t;
+typedef int64_t a2_gid_t;
 
 class RequestGroup {
 public:
@@ -82,23 +82,31 @@ public:
     SHUTDOWN_SIGNAL,
     USER_REQUEST
   };
+  enum State {
+    // Waiting in the reserved queue
+    STATE_WAITING,
+    // Download has begun
+    STATE_ACTIVE
+  };
 private:
-  static gid_t gidCounter_;
+  static a2_gid_t gidCounter_;
 
-  gid_t gid_;
+  a2_gid_t gid_;
+
+  int state_;
 
   SharedHandle<Option> option_;
 
-  size_t numConcurrentCommand_;
+  int numConcurrentCommand_;
 
   /**
    * This is the number of connections used in streaming protocol(http/ftp)
    */
-  unsigned int numStreamConnection_;
+  int numStreamConnection_;
 
-  unsigned int numStreamCommand_;
+  int numStreamCommand_;
 
-  unsigned int numCommand_;
+  int numCommand_;
 
   SharedHandle<SegmentMan> segmentMan_;
 
@@ -136,7 +144,7 @@ private:
 
   Time lastModifiedTime_;
 
-  unsigned int fileNotFoundCount_;
+  int fileNotFoundCount_;
 
   // Timeout used for HTTP/FTP downloads.
   time_t timeout_;
@@ -151,9 +159,9 @@ private:
   // just sits in memory.
   bool inMemoryDownload_;
 
-  unsigned int maxDownloadSpeedLimit_;
+  int maxDownloadSpeedLimit_;
 
-  unsigned int maxUploadSpeedLimit_;
+  int maxUploadSpeedLimit_;
 
   error_code::Value lastErrorCode_;
 
@@ -161,13 +169,13 @@ private:
   // example, downloads generated by PostDownloadHandler), this field
   // has the GID of generated RequestGroups. empty list means there is
   // no such RequestGroup.
-  std::vector<gid_t> followedByGIDs_;
+  std::vector<a2_gid_t> followedByGIDs_;
 
   // If this download is a part of another download(for example,
   // downloading torrent file described in Metalink file), this field
   // has the GID of parent RequestGroup. 0 means this is a parent
   // RequestGroup.
-  gid_t belongsToGID_;
+  a2_gid_t belongsToGID_;
 
   SharedHandle<MetadataInfo> metadataInfo_;
 
@@ -194,13 +202,13 @@ private:
   void removeDefunctControlFile
   (const SharedHandle<BtProgressInfoFile>& progressInfoFile);
 
-  bool isCheckIntegrityReady() const;
 public:
-  // The copy of option is stored in RequestGroup object.
   RequestGroup(const SharedHandle<Option>& option);
 
   ~RequestGroup();
 
+  bool isCheckIntegrityReady();
+
   const SharedHandle<SegmentMan>& getSegmentMan() const
   {
     return segmentMan_;
@@ -218,7 +226,7 @@ public:
                                 DownloadEngine* e, int numAdj);
 
   void createNextCommand(std::vector<Command*>& commands,
-                         DownloadEngine* e, unsigned int numCommand);
+                         DownloadEngine* e, int numCommand);
   
   void createNextCommand(std::vector<Command*>& commands, DownloadEngine* e);
 
@@ -230,9 +238,9 @@ public:
 
   std::string getFirstFilePath() const;
 
-  uint64_t getTotalLength() const;
+  int64_t getTotalLength() const;
 
-  uint64_t getCompletedLength() const;
+  int64_t getCompletedLength() const;
 
   /**
    * Compares expected filename with specified actualFilename.
@@ -241,22 +249,22 @@ public:
    */
   void validateFilename(const std::string& actualFilename) const;
 
-  void validateTotalLength(uint64_t expectedTotalLength,
-                           uint64_t actualTotalLength) const;
+  void validateTotalLength(int64_t expectedTotalLength,
+                           int64_t actualTotalLength) const;
 
-  void validateTotalLength(uint64_t actualTotalLength) const;
+  void validateTotalLength(int64_t actualTotalLength) const;
 
-  void setNumConcurrentCommand(unsigned int num)
+  void setNumConcurrentCommand(int num)
   {
     numConcurrentCommand_ = num;
   }
 
-  unsigned int getNumConcurrentCommand() const
+  int getNumConcurrentCommand() const
   {
     return numConcurrentCommand_;
   }
 
-  gid_t getGID() const
+  a2_gid_t getGID() const
   {
     return gid_;
   }
@@ -289,13 +297,13 @@ public:
 
   void decreaseStreamConnection();
 
-  unsigned int getNumConnection() const;
+  int getNumConnection() const;
 
   void increaseNumCommand();
 
   void decreaseNumCommand();
 
-  unsigned int getNumCommand() const
+  int getNumCommand() const
   {
     return numCommand_;
   }
@@ -461,22 +469,22 @@ public:
   // maxUploadSpeedLimit_ == 0. Otherwise returns false.
   bool doesUploadSpeedExceed();
 
-  unsigned int getMaxDownloadSpeedLimit() const
+  int getMaxDownloadSpeedLimit() const
   {
     return maxDownloadSpeedLimit_;
   }
 
-  void setMaxDownloadSpeedLimit(unsigned int speed)
+  void setMaxDownloadSpeedLimit(int speed)
   {
     maxDownloadSpeedLimit_ = speed;
   }
 
-  unsigned int getMaxUploadSpeedLimit() const
+  int getMaxUploadSpeedLimit() const
   {
     return maxUploadSpeedLimit_;
   }
 
-  void setMaxUploadSpeedLimit(unsigned int speed)
+  void setMaxUploadSpeedLimit(int speed)
   {
     maxUploadSpeedLimit_ = speed;
   }
@@ -486,6 +494,11 @@ public:
     lastErrorCode_ = code;
   }
 
+  error_code::Value getLastErrorCode() const
+  {
+    return lastErrorCode_;
+  }
+
   void saveControlFile() const;
 
   void removeControlFile() const;
@@ -503,17 +516,17 @@ public:
     }
   }
 
-  const std::vector<gid_t>& followedBy() const
+  const std::vector<a2_gid_t>& followedBy() const
   {
     return followedByGIDs_;
   }
 
-  void belongsTo(gid_t gid)
+  void belongsTo(a2_gid_t gid)
   {
     belongsToGID_ = gid;
   }
 
-  gid_t belongsTo() const
+  a2_gid_t belongsTo() const
   {
     return belongsToGID_;
   }
@@ -545,9 +558,19 @@ public:
     return metadataInfo_;
   }
 
+  int getState() const
+  {
+    return state_;
+  }
+
+  void setState(int state)
+  {
+    state_ = state;
+  }
+
   static void resetGIDCounter() { gidCounter_ = 0; }
 
-  static gid_t newGID();
+  static a2_gid_t newGID();
 };
 
 } // namespace aria2
