@@ -74,12 +74,19 @@
 #include "Triplet.h"
 #include "Signature.h"
 #include "OutputFile.h"
+#include "download_helper.h"
+#include "UriListParser.h"
+#include "SingletonHolder.h"
+#include "Notifier.h"
+#ifdef ENABLE_BITTORRENT
+#  include "bittorrent_helper.h"
+#endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
 RequestGroupMan::RequestGroupMan
 (const std::vector<SharedHandle<RequestGroup> >& requestGroups,
- unsigned int maxSimultaneousDownloads,
+ int maxSimultaneousDownloads,
  const Option* option)
   : reservedGroups_(requestGroups.begin(), requestGroups.end()),
     maxSimultaneousDownloads_(maxSimultaneousDownloads),
@@ -94,7 +101,9 @@ RequestGroupMan::RequestGroupMan
     removedErrorResult_(0),
     removedLastErrorResult_(error_code::FINISHED),
     maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT))
-{}
+{
+  addRequestGroupIndex(requestGroups);
+}
 
 RequestGroupMan::~RequestGroupMan() {}
 
@@ -116,6 +125,7 @@ void RequestGroupMan::addReservedGroup
 (const std::vector<SharedHandle<RequestGroup> >& groups)
 {
   requestQueueCheck();
+  addRequestGroupIndex(groups);
   reservedGroups_.insert(reservedGroups_.end(), groups.begin(), groups.end());
 }
 
@@ -123,6 +133,7 @@ void RequestGroupMan::addReservedGroup
 (const SharedHandle<RequestGroup>& group)
 {
   requestQueueCheck();
+  addRequestGroupIndex(group);
   reservedGroups_.push_back(group);
 }
 
@@ -130,6 +141,7 @@ void RequestGroupMan::insertReservedGroup
 (size_t pos, const std::vector<SharedHandle<RequestGroup> >& groups)
 {
   requestQueueCheck();
+  addRequestGroupIndex(groups);
   reservedGroups_.insert
     (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos),
      groups.begin(), groups.end());
@@ -139,10 +151,37 @@ void RequestGroupMan::insertReservedGroup
 (size_t pos, const SharedHandle<RequestGroup>& group)
 {
   requestQueueCheck();
+  addRequestGroupIndex(group);
   reservedGroups_.insert
     (reservedGroups_.begin()+std::min(reservedGroups_.size(), pos), group);
 }
 
+void RequestGroupMan::addRequestGroupIndex
+(const SharedHandle<RequestGroup>& group)
+{
+  assert(groupIndex_.count(group->getGID()) == 0);
+  groupIndex_[group->getGID()] = group;
+}
+
+void RequestGroupMan::addRequestGroupIndex
+(const std::vector<SharedHandle<RequestGroup> >& groups)
+{
+  for(std::vector<SharedHandle<RequestGroup> >::const_iterator i =
+        groups.begin(); i != groups.end(); ++i) {
+    addRequestGroupIndex(*i);
+  }
+}
+
+namespace {
+void removeRequestGroupIndex
+(std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex,
+ const SharedHandle<RequestGroup>& group)
+{
+  assert(groupIndex.count(group->getGID()) == 1);
+  groupIndex.erase(group->getGID());
+}
+} // namespace
+
 size_t RequestGroupMan::countRequestGroup() const
 {
   return requestGroups_.size();
@@ -173,24 +212,41 @@ Iterator findByGID(Iterator first, Iterator last, a2_gid_t gid)
 SharedHandle<RequestGroup>
 RequestGroupMan::findRequestGroup(a2_gid_t gid) const
 {
-  std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-    findByGID(requestGroups_.begin(), requestGroups_.end(), gid);
-  if(i == requestGroups_.end()) {
-    return SharedHandle<RequestGroup>();
+  SharedHandle<RequestGroup> res = findGroup(gid);
+  if(res) {
+    if(res->getState() == RequestGroup::STATE_ACTIVE) {
+      return res;
+    } else {
+      return SharedHandle<RequestGroup>();
+    }
   } else {
-    return *i;
+    return res;
   }
 }
 
 SharedHandle<RequestGroup>
 RequestGroupMan::findReservedGroup(a2_gid_t gid) const
 {
-  std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-    findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
-  if(i == reservedGroups_.end()) {
-    return SharedHandle<RequestGroup>();
+  SharedHandle<RequestGroup> res = findGroup(gid);
+  if(res) {
+    if(res->getState() == RequestGroup::STATE_WAITING) {
+      return res;
+    } else {
+      return SharedHandle<RequestGroup>();
+    }
   } else {
-    return *i;
+    return res;
+  }
+}
+
+SharedHandle<RequestGroup> RequestGroupMan::findGroup(a2_gid_t gid) const
+{
+  std::map<a2_gid_t, SharedHandle<RequestGroup> >::const_iterator i =
+    groupIndex_.find(gid);
+  if(i != groupIndex_.end()) {
+    return (*i).second;
+  } else {
+    return SharedHandle<RequestGroup>();
   }
 }
 
@@ -200,9 +256,7 @@ size_t RequestGroupMan::changeReservedGroupPosition
   std::deque<SharedHandle<RequestGroup> >::iterator i =
     findByGID(reservedGroups_.begin(), reservedGroups_.end(), gid);
   if(i == reservedGroups_.end()) {
-    throw DL_ABORT_EX
-      (fmt("GID#%s not found in the waiting queue.",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("GID#%" PRId64 " not found in the waiting queue.", gid));
   }
   SharedHandle<RequestGroup> rg = *i;
   const size_t maxPos = reservedGroups_.size()-1;
@@ -245,6 +299,7 @@ bool RequestGroupMan::removeReservedGroup(a2_gid_t gid)
   if(i == reservedGroups_.end()) {
     return false;
   } else {
+    removeRequestGroupIndex(groupIndex_, *i);
     reservedGroups_.erase(i);
     return true;
   }
@@ -252,6 +307,16 @@ bool RequestGroupMan::removeReservedGroup(a2_gid_t gid)
 
 namespace {
 
+void notifyDownloadEvent
+(const std::string& event, const SharedHandle<RequestGroup>& group)
+{
+  SingletonHolder<Notifier>::instance()->notifyDownloadEvent(event, group);
+}
+
+} // namespace
+
+namespace {
+
 void executeStopHook
 (const SharedHandle<RequestGroup>& group,
  const Option* option,
@@ -267,6 +332,14 @@ void executeStopHook
   } else if(!option->blank(PREF_ON_DOWNLOAD_STOP)) {
     util::executeHookByOptName(group, option, PREF_ON_DOWNLOAD_STOP);
   }
+  if(result == error_code::FINISHED) {
+    notifyDownloadEvent(Notifier::ON_DOWNLOAD_COMPLETE, group);
+  } else if(result != error_code::IN_PROGRESS &&
+            result != error_code::REMOVED) {
+    notifyDownloadEvent(Notifier::ON_DOWNLOAD_ERROR, group);
+  } else {
+    notifyDownloadEvent(Notifier::ON_DOWNLOAD_STOP, group);
+  }
 }
 
 } // namespace
@@ -277,7 +350,7 @@ private:
   DownloadEngine* e_;
   std::deque<SharedHandle<DownloadResult> >& downloadResults_;
   std::deque<SharedHandle<RequestGroup> >& reservedGroups_;
-  Logger* logger_;
+  std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex_;
 
   void saveSignature(const SharedHandle<RequestGroup>& group)
   {
@@ -298,10 +371,12 @@ public:
   ProcessStoppedRequestGroup
   (DownloadEngine* e,
    std::deque<SharedHandle<DownloadResult> >& downloadResults,
-   std::deque<SharedHandle<RequestGroup> >& reservedGroups)
+   std::deque<SharedHandle<RequestGroup> >& reservedGroups,
+   std::map<a2_gid_t, SharedHandle<RequestGroup> >& groupIndex)
     : e_(e),
       downloadResults_(downloadResults),
-      reservedGroups_(reservedGroups)
+      reservedGroups_(reservedGroups),
+      groupIndex_(groupIndex)
   {}
 
   void operator()(const SharedHandle<RequestGroup>& group)
@@ -339,10 +414,33 @@ public:
                    static_cast<unsigned long>(nextGroups.size())));
             e_->getRequestGroupMan()->insertReservedGroup(0, nextGroups);
           }
+#ifdef ENABLE_BITTORRENT
+          // For in-memory download (e.g., Magnet URI), the
+          // FileEntry::getPath() does not return actual file path, so
+          // we don't remove it.
+          if(group->getOption()->getAsBool(PREF_BT_REMOVE_UNSELECTED_FILE) &&
+             !group->inMemoryDownload() &&
+             dctx->hasAttribute(bittorrent::BITTORRENT)) {
+            A2_LOG_INFO(fmt(MSG_REMOVING_UNSELECTED_FILE, group->getGID()));
+            const std::vector<SharedHandle<FileEntry> >& files =
+              dctx->getFileEntries();
+            for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
+                  files.begin(), eoi = files.end(); i != eoi; ++i) {
+              if(!(*i)->isRequested()) {
+                if(File((*i)->getPath()).remove()) {
+                  A2_LOG_INFO(fmt(MSG_FILE_REMOVED, (*i)->getPath().c_str()));
+                } else {
+                  A2_LOG_INFO(fmt(MSG_FILE_COULD_NOT_REMOVED,
+                                  (*i)->getPath().c_str()));
+                }
+              }
+            }
+          }
+#endif // ENABLE_BITTORRENT
         } else {
           A2_LOG_NOTICE
-            (fmt("Download GID#%s not complete: %s",
-                 util::itos(group->getGID()).c_str(),
+            (fmt(_("Download GID#%" PRId64 " not complete: %s"),
+                 group->getGID(),
                  group->getDownloadContext()->getBasePath().c_str()));
           group->saveControlFile();
         }
@@ -350,11 +448,13 @@ public:
         A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, ex);
       }
       if(group->isPauseRequested()) {
+        group->setState(RequestGroup::STATE_WAITING);
         reservedGroups_.push_front(group);
         group->releaseRuntimeResource(e_);
         group->setForceHaltRequested(false);
-        util::executeHookByOptName
-          (group, e_->getOption(), PREF_ON_DOWNLOAD_PAUSE);
+        util::executeHookByOptName(group, e_->getOption(),
+                                   PREF_ON_DOWNLOAD_PAUSE);
+        notifyDownloadEvent(Notifier::ON_DOWNLOAD_PAUSE, group);
         // TODO Should we have to prepend spend uris to remaining uris
         // in case PREF_REUSE_URI is disabed?
       } else {
@@ -362,6 +462,7 @@ public:
         e_->getRequestGroupMan()->addDownloadResult(dr);
         executeStopHook(group, e_->getOption(), dr->result);
         group->releaseRuntimeResource(e_);
+        removeRequestGroupIndex(groupIndex_, group);
       }
     }
   }
@@ -407,7 +508,7 @@ public:
           }
         }
       }
-    }    
+    }
   }
 };
 } // namespace
@@ -435,7 +536,7 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 
   std::for_each(requestGroups_.begin(), requestGroups_.end(),
                 ProcessStoppedRequestGroup
-                (e, downloadResults_, reservedGroups_));
+                (e, downloadResults_, reservedGroups_, groupIndex_));
   std::deque<SharedHandle<RequestGroup> >::iterator i =
     std::remove_if(requestGroups_.begin(),
                    requestGroups_.end(),
@@ -454,16 +555,17 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 void RequestGroupMan::configureRequestGroup
 (const SharedHandle<RequestGroup>& requestGroup) const
 {
-  const std::string& uriSelectorValue = option_->get(PREF_URI_SELECTOR);
+  const std::string& uriSelectorValue =
+    requestGroup->getOption()->get(PREF_URI_SELECTOR);
+  SharedHandle<URISelector> sel;
   if(uriSelectorValue == V_FEEDBACK) {
-    SharedHandle<URISelector> sel(new FeedbackURISelector(serverStatMan_));
-    requestGroup->setURISelector(sel);
+    sel.reset(new FeedbackURISelector(serverStatMan_));
   } else if(uriSelectorValue == V_INORDER) {
-    SharedHandle<URISelector> sel(new InorderURISelector());
-    requestGroup->setURISelector(sel);
+    sel.reset(new InorderURISelector());
   } else if(uriSelectorValue == V_ADAPTIVE) {
-    SharedHandle<URISelector> sel(new AdaptiveURISelector(serverStatMan_,
-                                                          requestGroup.get()));
+    sel.reset(new AdaptiveURISelector(serverStatMan_, requestGroup.get()));
+  }
+  if(sel) {
     requestGroup->setURISelector(sel);
   }
 }
@@ -480,13 +582,28 @@ void createInitialCommand(const SharedHandle<RequestGroup>& requestGroup,
 void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
 {
   removeStoppedGroup(e);
-  if(maxSimultaneousDownloads_ <= requestGroups_.size()) {
+  if(static_cast<size_t>(maxSimultaneousDownloads_) <= requestGroups_.size()) {
     return;
   }
   std::vector<SharedHandle<RequestGroup> > temp;
-  unsigned int count = 0;
-  size_t num = maxSimultaneousDownloads_-requestGroups_.size();
-  while(count < num && !reservedGroups_.empty()) {
+  int count = 0;
+  int num = maxSimultaneousDownloads_-requestGroups_.size();
+  while(count < num && (uriListParser_ || !reservedGroups_.empty())) {
+    if(uriListParser_ && reservedGroups_.empty()) {
+      std::vector<SharedHandle<RequestGroup> > groups;
+      bool ok = createRequestGroupFromUriListParser(groups, option_,
+                                                    uriListParser_.get());
+      if(ok) {
+        addRequestGroupIndex(groups);
+        reservedGroups_.insert(reservedGroups_.end(), groups.begin(),
+                               groups.end());
+      } else {
+        uriListParser_.reset();
+        if(reservedGroups_.empty()) {
+          break;
+        }
+      }
+    }
     SharedHandle<RequestGroup> groupToAdd = reservedGroups_.front();
     reservedGroups_.pop_front();
     std::vector<Command*> commands;
@@ -505,6 +622,7 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       if(commands.empty()) {
         requestQueueCheck();
       }
+      groupToAdd->setState(RequestGroup::STATE_ACTIVE);
       requestGroups_.push_back(groupToAdd);
       ++count;
       e->addCommand(commands);
@@ -518,11 +636,13 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       groupToAdd->setLastErrorCode(ex.getErrorCode());
       // We add groupToAdd to e in order to it is processed in
       // removeStoppedGroup().
+      groupToAdd->setState(RequestGroup::STATE_ACTIVE);
       requestGroups_.push_back(groupToAdd);
       requestQueueCheck();
     }
-    util::executeHookByOptName
-      (groupToAdd, e->getOption(), PREF_ON_DOWNLOAD_START);
+    util::executeHookByOptName(groupToAdd, e->getOption(),
+                               PREF_ON_DOWNLOAD_START);
+    notifyDownloadEvent(Notifier::ON_DOWNLOAD_START, groupToAdd);
   }
   if(!temp.empty()) {
     reservedGroups_.insert(reservedGroups_.begin(), temp.begin(), temp.end());
@@ -538,7 +658,7 @@ void RequestGroupMan::save()
 {
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
         requestGroups_.begin(), eoi = requestGroups_.end(); itr != eoi; ++itr) {
-    if((*itr)->allDownloadFinished() && 
+    if((*itr)->allDownloadFinished() &&
        !(*itr)->getDownloadContext()->isChecksumVerificationNeeded()) {
       (*itr)->removeControlFile();
     } else {
@@ -561,10 +681,10 @@ void RequestGroupMan::closeFile()
 
 RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
 {
-  size_t finished = 0;
-  size_t error = removedErrorResult_;
-  size_t inprogress = 0;
-  size_t removed = 0;
+  int finished = 0;
+  int error = removedErrorResult_;
+  int inprogress = 0;
+  int removed = 0;
   error_code::Value lastError = removedLastErrorResult_;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
@@ -650,16 +770,16 @@ void RequestGroupMan::showDownloadResults(OutputFile& o, bool full) const
   if(ok > 0 || err > 0 || inpr > 0 || rm > 0) {
     o.printf("\n%s\n", _("Status Legend:"));
     if(ok > 0) {
-      o.write(" (OK):download completed.");
+      o.write(_("(OK):download completed."));
     }
     if(err > 0) {
-      o.write("(ERR):error occurred.");
+      o.write(_("(ERR):error occurred."));
     }
     if(inpr > 0) {
-      o.write("(INPR):download in-progress.");
+      o.write(_("(INPR):download in-progress."));
     }
     if(rm > 0) {
-      o.write("(RM):download removed.");
+      o.write(_("(RM):download removed."));
     }
     o.write("\n");
   }
@@ -692,8 +812,8 @@ void RequestGroupMan::formatDownloadResultFull
 {
   BitfieldMan bt(downloadResult->pieceLength, downloadResult->totalLength);
   bt.setBitfield(reinterpret_cast<const unsigned char*>
-                 (util::fromHex(downloadResult->bitfieldStr).data()),
-                 downloadResult->bitfieldStr.size()/2);
+                 (downloadResult->bitfield.data()),
+                 downloadResult->bitfield.size());
   bool head = true;
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
     downloadResult->fileEntries;
@@ -709,10 +829,10 @@ void RequestGroupMan::formatDownloadResultFull
     } else {
       o << "   |    |           |";
     }
-    if((*i)->getLength() == 0 || downloadResult->bitfieldStr.empty()) {
+    if((*i)->getLength() == 0 || downloadResult->bitfield.empty()) {
       o << "  -|";
     } else {
-      uint64_t completedLength =
+      int64_t completedLength =
         bt.getOffsetCompletedLength((*i)->getOffset(), (*i)->getLength());
       o << std::setw(3) << 100*completedLength/(*i)->getLength() << "|";
     }
@@ -851,7 +971,7 @@ void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
       ++removedErrorResult_;
     }
   } else {
-    size_t curSize = downloadResults_.size();
+    int curSize = downloadResults_.size();
     if(curSize >= maxDownloadResult_) {
       std::deque<SharedHandle<DownloadResult> >::iterator last =
         downloadResults_.begin()+curSize-maxDownloadResult_+1;
@@ -861,7 +981,7 @@ void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
           removedLastErrorResult_ = (*i)->result;
           ++removedErrorResult_;
         }
-      }        
+      }
       downloadResults_.erase(downloadResults_.begin(), last);
     }
     downloadResults_.push_back(dr);
@@ -934,9 +1054,9 @@ void RequestGroupMan::getUsedHosts
   std::vector<Triplet<size_t, int, std::string> > tempHosts;
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
         requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
-    const std::deque<SharedHandle<Request> >& inFlightReqs =
+    const FileEntry::InFlightRequestSet& inFlightReqs =
       (*i)->getDownloadContext()->getFirstFileEntry()->getInFlightRequests();
-    for(std::deque<SharedHandle<Request> >::const_iterator j =
+    for(FileEntry::InFlightRequestSet::iterator j =
           inFlightReqs.begin(), eoj = inFlightReqs.end(); j != eoj; ++j) {
       uri::UriStruct us;
       if(uri::parse(us, (*j)->getUri())) {
@@ -963,4 +1083,10 @@ void RequestGroupMan::getUsedHosts
                  std::back_inserter(usedHosts), Tuple2Pair<1, 3>());
 }
 
+void RequestGroupMan::setUriListParser
+(const SharedHandle<UriListParser>& uriListParser)
+{
+  uriListParser_ = uriListParser;
+}
+
 } // namespace aria2
