@@ -36,6 +36,7 @@
 
 #include <cassert>
 #include <algorithm>
+#include <sstream>
 
 #include "Logger.h"
 #include "LogFactory.h"
@@ -63,7 +64,7 @@
 #include "SegmentMan.h"
 #include "TimedHaltCommand.h"
 #include "PeerStat.h"
-#include "Base64.h"
+#include "base64.h"
 #include "BitfieldMan.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigest.h"
@@ -166,18 +167,6 @@ addRequestGroup(const SharedHandle<RequestGroup>& group,
 }
 } // namespace
 
-namespace {
-SharedHandle<RequestGroup>
-findRequestGroup(const SharedHandle<RequestGroupMan>& rgman, a2_gid_t gid)
-{
-  SharedHandle<RequestGroup> group = rgman->findRequestGroup(gid);
-  if(!group) {
-    group = rgman->findReservedGroup(gid);
-  }
-  return group;
-}
-} // namespace
-
 namespace {
 bool checkPosParam(const Integer* posParam)
 {
@@ -230,7 +219,7 @@ SharedHandle<ValueBase> AddUriRpcMethod::process
   }
 
   SharedHandle<Option> requestOption(new Option(*e->getOption()));
-  gatherRequestOption(requestOption, optsParam);
+  gatherRequestOption(requestOption.get(), optsParam);
 
   bool posGiven = checkPosParam(posParam);
   size_t pos = posGiven ? posParam->i() : 0;
@@ -270,14 +259,16 @@ SharedHandle<ValueBase> AddTorrentRpcMethod::process
 
   SharedHandle<String> tempTorrentParam;
   if(req.jsonRpc) {
-    tempTorrentParam = String::g(Base64::decode(torrentParam->s()));
+    tempTorrentParam = String::g
+      (base64::decode(torrentParam->s().begin(),
+                      torrentParam->s().end()));
     torrentParam = tempTorrentParam.get();
   }
   std::vector<std::string> uris;
   extractUris(std::back_inserter(uris), urisParam);
 
   SharedHandle<Option> requestOption(new Option(*e->getOption()));
-  gatherRequestOption(requestOption, optsParam);
+  gatherRequestOption(requestOption.get(), optsParam);
 
   bool posGiven = checkPosParam(posParam);
   size_t pos = posGiven ? posParam->i() : 0;
@@ -290,13 +281,13 @@ SharedHandle<ValueBase> AddTorrentRpcMethod::process
   if(util::saveAs(filename, torrentParam->s(), true)) {
     A2_LOG_INFO(fmt("Uploaded torrent data was saved as %s", filename.c_str()));
     requestOption->put(PREF_TORRENT_FILE, filename);
-    createRequestGroupForBitTorrent(result, requestOption, uris);
   } else {
     A2_LOG_INFO(fmt("Uploaded torrent data was not saved."
                     " Failed to write file %s", filename.c_str()));
-    createRequestGroupForBitTorrent(result, requestOption,
-                                    uris, torrentParam->s());
+    filename.clear();
   }
+  createRequestGroupForBitTorrent(result, requestOption, uris, filename,
+                                  torrentParam->s());
 
   if(!result.empty()) {
     return addRequestGroup(result.front(), e, posGiven, pos);
@@ -316,11 +307,13 @@ SharedHandle<ValueBase> AddMetalinkRpcMethod::process
 
   SharedHandle<String> tempMetalinkParam;
   if(req.jsonRpc) {
-    tempMetalinkParam = String::g(Base64::decode(metalinkParam->s()));
+    tempMetalinkParam = String::g
+      (base64::decode(metalinkParam->s().begin(),
+                      metalinkParam->s().end()));
     metalinkParam = tempMetalinkParam.get();
   }
   SharedHandle<Option> requestOption(new Option(*e->getOption()));
-  gatherRequestOption(requestOption, optsParam);
+  gatherRequestOption(requestOption.get(), optsParam);
 
   bool posGiven = checkPosParam(posParam);
   size_t pos = posGiven ? posParam->i() : 0;
@@ -371,29 +364,25 @@ SharedHandle<ValueBase> removeDownload
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    e->getRequestGroupMan()->findRequestGroup(gid);
-  if(!group) {
-    group = e->getRequestGroupMan()->findReservedGroup(gid);
-    if(!group) {
-      throw DL_ABORT_EX
-        (fmt("Active Download not found for GID#%s",
-             util::itos(gid).c_str()));
-    }
-    if(group->isDependencyResolved()) {
-      e->getRequestGroupMan()->removeReservedGroup(gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
+  if(group) {
+    if(group->getState() == RequestGroup::STATE_ACTIVE) {
+      if(forceRemove) {
+        group->setForceHaltRequested(true, RequestGroup::USER_REQUEST);
+      } else {
+        group->setHaltRequested(true, RequestGroup::USER_REQUEST);
+      }
+      e->setRefreshInterval(0);
     } else {
-      throw DL_ABORT_EX
-        (fmt("GID#%s cannot be removed now",
-             util::itos(gid).c_str()));
+      if(group->isDependencyResolved()) {
+        e->getRequestGroupMan()->removeReservedGroup(gid);
+      } else {
+        throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be removed now", gid));
+      }
     }
   } else {
-    if(forceRemove) {
-      group->setForceHaltRequested(true, RequestGroup::USER_REQUEST);
-    } else {
-      group->setHaltRequested(true, RequestGroup::USER_REQUEST);
-    }
-    e->setRefreshInterval(0);
+    throw DL_ABORT_EX(fmt("Active Download not found for GID#%" PRId64,
+                          gid));
   }
   return createGIDResponse(gid);
 }
@@ -444,21 +433,15 @@ SharedHandle<ValueBase> pauseDownload
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  bool reserved = false;
-  SharedHandle<RequestGroup> group =
-    e->getRequestGroupMan()->findRequestGroup(gid);
-  if(!group) {
-    reserved = true;
-    group = e->getRequestGroupMan()->findReservedGroup(gid);
-  }
-  if(group && pauseRequestGroup(group, reserved, forcePause)) {
-    e->setRefreshInterval(0);
-    return createGIDResponse(gid);
-  } else {
-    throw DL_ABORT_EX
-      (fmt("GID#%s cannot be paused now",
-           util::itos(gid).c_str()));
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
+  if(group) {
+    bool reserved = group->getState() == RequestGroup::STATE_WAITING;
+    if(pauseRequestGroup(group, reserved, forcePause)) {
+      e->setRefreshInterval(0);
+      return createGIDResponse(gid);
+    }
   }
+  throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be paused now", gid));
 }
 } // namespace
 
@@ -518,12 +501,11 @@ SharedHandle<ValueBase> UnpauseRpcMethod::process
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    e->getRequestGroupMan()->findReservedGroup(gid);
-  if(!group || !group->isPauseRequested()) {
-    throw DL_ABORT_EX
-      (fmt("GID#%s cannot be unpaused now",
-           util::itos(gid).c_str()));
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
+  if(!group ||
+     group->getState() != RequestGroup::STATE_WAITING ||
+     !group->isPauseRequested()) {
+    throw DL_ABORT_EX(fmt("GID#%" PRId64 " cannot be unpaused now", gid));
   } else {
     group->setPauseRequested(false);
     e->getRequestGroupMan()->requestQueueCheck();    
@@ -587,10 +569,10 @@ void createFileEntry
     entry->put(KEY_INDEX, util::uitos(index));
     entry->put(KEY_PATH, (*first)->getPath());
     entry->put(KEY_SELECTED, (*first)->isRequested()?VLB_TRUE:VLB_FALSE);
-    entry->put(KEY_LENGTH, util::uitos((*first)->getLength()));
-    uint64_t completedLength = bf->getOffsetCompletedLength
+    entry->put(KEY_LENGTH, util::itos((*first)->getLength()));
+    int64_t completedLength = bf->getOffsetCompletedLength
       ((*first)->getOffset(), (*first)->getLength());
-    entry->put(KEY_COMPLETED_LENGTH, util::uitos(completedLength));
+    entry->put(KEY_COMPLETED_LENGTH, util::itos(completedLength));
 
     SharedHandle<List> uriList = List::g();
     createUriEntry(uriList, *first);
@@ -605,11 +587,10 @@ template<typename InputIterator>
 void createFileEntry
 (const SharedHandle<List>& files,
  InputIterator first, InputIterator last,
- uint64_t totalLength,
- size_t pieceLength,
- const std::string& bitfieldStr)
+ int64_t totalLength,
+ int32_t pieceLength,
+ const std::string& bitfield)
 {
-  std::string bitfield = util::fromHex(bitfieldStr);
   BitfieldMan bf(pieceLength, totalLength);
   bf.setBitfield(reinterpret_cast<const unsigned char*>(bitfield.data()),
                  bitfield.size());
@@ -622,8 +603,8 @@ template<typename InputIterator>
 void createFileEntry
 (const SharedHandle<List>& files,
  InputIterator first, InputIterator last,
- uint64_t totalLength,
- size_t pieceLength,
+ int64_t totalLength,
+ int32_t pieceLength,
  const SharedHandle<PieceStorage>& ps)
 {
   BitfieldMan bf(pieceLength, totalLength);
@@ -653,26 +634,26 @@ void gatherProgressCommon
   }
   if(requested_key(keys, KEY_TOTAL_LENGTH)) {
     // This is "filtered" total length if --select-file is used.
-    entryDict->put(KEY_TOTAL_LENGTH, util::uitos(group->getTotalLength()));
+    entryDict->put(KEY_TOTAL_LENGTH, util::itos(group->getTotalLength()));
   }
   if(requested_key(keys, KEY_COMPLETED_LENGTH)) {
     // This is "filtered" total length if --select-file is used.
     entryDict->put
-      (KEY_COMPLETED_LENGTH,util::uitos(group->getCompletedLength()));
+      (KEY_COMPLETED_LENGTH,util::itos(group->getCompletedLength()));
   }
   TransferStat stat = group->calculateStat();
   if(requested_key(keys, KEY_DOWNLOAD_SPEED)) {
-    entryDict->put(KEY_DOWNLOAD_SPEED, util::uitos(stat.getDownloadSpeed()));
+    entryDict->put(KEY_DOWNLOAD_SPEED, util::itos(stat.getDownloadSpeed()));
   }
   if(requested_key(keys, KEY_UPLOAD_SPEED)) {
-    entryDict->put(KEY_UPLOAD_SPEED, util::uitos(stat.getUploadSpeed()));
+    entryDict->put(KEY_UPLOAD_SPEED, util::itos(stat.getUploadSpeed()));
   }
   if(requested_key(keys, KEY_UPLOAD_LENGTH)) {
     entryDict->put
-      (KEY_UPLOAD_LENGTH, util::uitos(stat.getAllTimeUploadLength()));
+      (KEY_UPLOAD_LENGTH, util::itos(stat.getAllTimeUploadLength()));
   }
   if(requested_key(keys, KEY_CONNECTIONS)) {
-    entryDict->put(KEY_CONNECTIONS, util::uitos(group->getNumConnection()));
+    entryDict->put(KEY_CONNECTIONS, util::itos(group->getNumConnection()));
   }
   if(requested_key(keys, KEY_BITFIELD)) {
     if(ps) {
@@ -684,7 +665,7 @@ void gatherProgressCommon
   }
   const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
   if(requested_key(keys, KEY_PIECE_LENGTH)) {
-    entryDict->put(KEY_PIECE_LENGTH, util::uitos(dctx->getPieceLength()));
+    entryDict->put(KEY_PIECE_LENGTH, util::itos(dctx->getPieceLength()));
   }
   if(requested_key(keys, KEY_NUM_PIECES)) {
     entryDict->put(KEY_NUM_PIECES, util::uitos(dctx->getNumPieces()));
@@ -754,7 +735,7 @@ namespace {
 void gatherProgressBitTorrent
 (const SharedHandle<Dict>& entryDict,
  const SharedHandle<TorrentAttribute>& torrentAttrs,
- const BtObject& btObject,
+ const SharedHandle<BtObject>& btObject,
  const std::vector<std::string>& keys)
 {
   if(requested_key(keys, KEY_INFO_HASH)) {
@@ -766,10 +747,10 @@ void gatherProgressBitTorrent
     entryDict->put(KEY_BITTORRENT, btDict);
   }
   if(requested_key(keys, KEY_NUM_SEEDERS)) {
-    if(btObject.isNull()) {
+    if(!btObject) {
       entryDict->put(KEY_NUM_SEEDERS, VLB_ZERO);
     } else {
-      SharedHandle<PeerStorage> peerStorage = btObject.peerStorage_;
+      const SharedHandle<PeerStorage>& peerStorage = btObject->peerStorage;
       assert(peerStorage);
       std::vector<SharedHandle<Peer> > peers;
       peerStorage->getActivePeers(peers);
@@ -802,8 +783,8 @@ void gatherPeer
     peerEntry->put(KEY_AM_CHOKING, (*i)->amChoking()?VLB_TRUE:VLB_FALSE);
     peerEntry->put(KEY_PEER_CHOKING, (*i)->peerChoking()?VLB_TRUE:VLB_FALSE);
     TransferStat stat = ps->getTransferStatFor(*i);
-    peerEntry->put(KEY_DOWNLOAD_SPEED, util::uitos(stat.getDownloadSpeed()));
-    peerEntry->put(KEY_UPLOAD_SPEED, util::uitos(stat.getUploadSpeed()));
+    peerEntry->put(KEY_DOWNLOAD_SPEED, util::itos(stat.getDownloadSpeed()));
+    peerEntry->put(KEY_UPLOAD_SPEED, util::itos(stat.getUploadSpeed()));
     peerEntry->put(KEY_SEEDER, (*i)->isSeeder()?VLB_TRUE:VLB_FALSE);
     peers->append(peerEntry);
   }
@@ -823,7 +804,8 @@ void gatherProgress
   if(group->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) {
     SharedHandle<TorrentAttribute> torrentAttrs =
       bittorrent::getTorrentAttrs(group->getDownloadContext());
-    BtObject btObject = e->getBtRegistry()->get(group->getGID());
+    const SharedHandle<BtObject>& btObject =
+      e->getBtRegistry()->get(group->getGID());
     gatherProgressBitTorrent(entryDict, torrentAttrs, btObject, keys);
   }
 #endif // ENABLE_BITTORRENT
@@ -868,21 +850,21 @@ void gatherStoppedDownload
   if(requested_key(keys, KEY_FILES)) {
     SharedHandle<List> files = List::g();
     createFileEntry(files, ds->fileEntries.begin(), ds->fileEntries.end(),
-                    ds->totalLength, ds->pieceLength, ds->bitfieldStr);
+                    ds->totalLength, ds->pieceLength, ds->bitfield);
     entryDict->put(KEY_FILES, files);
   }
   if(requested_key(keys, KEY_TOTAL_LENGTH)) {
-    entryDict->put(KEY_TOTAL_LENGTH, util::uitos(ds->totalLength));
+    entryDict->put(KEY_TOTAL_LENGTH, util::itos(ds->totalLength));
   }
   if(requested_key(keys, KEY_COMPLETED_LENGTH)) {
-    entryDict->put(KEY_COMPLETED_LENGTH, util::uitos(ds->completedLength));
+    entryDict->put(KEY_COMPLETED_LENGTH, util::itos(ds->completedLength));
   }
   if(requested_key(keys, KEY_UPLOAD_LENGTH)) {
-    entryDict->put(KEY_UPLOAD_LENGTH, util::uitos(ds->uploadLength));
+    entryDict->put(KEY_UPLOAD_LENGTH, util::itos(ds->uploadLength));
   }
   if(requested_key(keys, KEY_BITFIELD)) {
-    if(!ds->bitfieldStr.empty()) {
-      entryDict->put(KEY_BITFIELD, ds->bitfieldStr);
+    if(!ds->bitfield.empty()) {
+      entryDict->put(KEY_BITFIELD, util::toHex(ds->bitfield));
     }
   }
   if(requested_key(keys, KEY_DOWNLOAD_SPEED)) {
@@ -891,16 +873,16 @@ void gatherStoppedDownload
   if(requested_key(keys, KEY_UPLOAD_SPEED)) {
     entryDict->put(KEY_UPLOAD_SPEED, VLB_ZERO);
   }
-  if(!ds->infoHashStr.empty()) {
+  if(!ds->infoHash.empty()) {
     if(requested_key(keys, KEY_INFO_HASH)) {
-      entryDict->put(KEY_INFO_HASH, ds->infoHashStr);
+      entryDict->put(KEY_INFO_HASH, util::toHex(ds->infoHash));
     }
     if(requested_key(keys, KEY_NUM_SEEDERS)) {
       entryDict->put(KEY_NUM_SEEDERS, VLB_ZERO);
     }
   }
   if(requested_key(keys, KEY_PIECE_LENGTH)) {
-    entryDict->put(KEY_PIECE_LENGTH, util::uitos(ds->pieceLength));
+    entryDict->put(KEY_PIECE_LENGTH, util::itos(ds->pieceLength));
   }
   if(requested_key(keys, KEY_NUM_PIECES)) {
     entryDict->put(KEY_NUM_PIECES, util::uitos(ds->numPieces));
@@ -920,18 +902,16 @@ SharedHandle<ValueBase> GetFilesRpcMethod::process
 
   a2_gid_t gid = str2Gid(gidParam);
   SharedHandle<List> files = List::g();
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
     SharedHandle<DownloadResult> dr =
       e->getRequestGroupMan()->findDownloadResult(gid);
     if(!dr) {
-      throw DL_ABORT_EX
-        (fmt("No file data is available for GID#%s",
-             util::itos(gid).c_str()));
+      throw DL_ABORT_EX(fmt("No file data is available for GID#%" PRId64 "",
+                            gid));
     } else {
       createFileEntry(files, dr->fileEntries.begin(), dr->fileEntries.end(),
-                      dr->totalLength, dr->pieceLength, dr->bitfieldStr);
+                      dr->totalLength, dr->pieceLength, dr->bitfield);
     }
   } else {
     const SharedHandle<PieceStorage>& ps = group->getPieceStorage();
@@ -952,12 +932,9 @@ SharedHandle<ValueBase> GetUrisRpcMethod::process
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX
-      (fmt("No URI data is available for GID#%s",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("No URI data is available for GID#%" PRId64, gid));
   }
   SharedHandle<List> uriList = List::g();
   // TODO Current implementation just returns first FileEntry's URIs.
@@ -974,18 +951,16 @@ SharedHandle<ValueBase> GetPeersRpcMethod::process
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX
-      (fmt("No peer data is available for GID#%s",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("No peer data is available for GID#%" PRId64, gid));
   }
   SharedHandle<List> peers = List::g();
-  BtObject btObject = e->getBtRegistry()->get(group->getGID());
-  if(!btObject.isNull()) {
-    assert(btObject.peerStorage_);
-    gatherPeer(peers, btObject.peerStorage_);
+  const SharedHandle<BtObject>& btObject =
+    e->getBtRegistry()->get(group->getGID());
+  if(btObject) {
+    assert(btObject->peerStorage);
+    gatherPeer(peers, btObject->peerStorage);
   }
   return peers;
 }
@@ -1001,34 +976,27 @@ SharedHandle<ValueBase> TellStatusRpcMethod::process
   std::vector<std::string> keys;
   toStringList(std::back_inserter(keys), keysParam);
 
-  SharedHandle<RequestGroup> group =
-    e->getRequestGroupMan()->findRequestGroup(gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
 
   SharedHandle<Dict> entryDict = Dict::g();
   if(!group) {
-    group = e->getRequestGroupMan()->findReservedGroup(gid);
-    if(!group) {
-      SharedHandle<DownloadResult> ds =
-        e->getRequestGroupMan()->findDownloadResult(gid);
-      if(!ds) {
-        throw DL_ABORT_EX
-          (fmt("No such download for GID#%s",
-               util::itos(gid).c_str()));
-      }
-      gatherStoppedDownload(entryDict, ds, keys);
-    } else {
-      if(requested_key(keys, KEY_STATUS)) {
+    SharedHandle<DownloadResult> ds =
+      e->getRequestGroupMan()->findDownloadResult(gid);
+    if(!ds) {
+      throw DL_ABORT_EX(fmt("No such download for GID#%" PRId64 "", gid));
+    }
+    gatherStoppedDownload(entryDict, ds, keys);
+  } else {
+    if(requested_key(keys, KEY_STATUS)) {
+      if(group->getState() == RequestGroup::STATE_ACTIVE) {
+        entryDict->put(KEY_STATUS, VLB_ACTIVE);
+      } else {
         if(group->isPauseRequested()) {
           entryDict->put(KEY_STATUS, VLB_PAUSED);
         } else {
           entryDict->put(KEY_STATUS, VLB_WAITING);
         }
       }
-      gatherProgress(entryDict, group, e, keys);
-    }
-  } else {
-    if(requested_key(keys, KEY_STATUS)) {
-      entryDict->put(KEY_STATUS, VLB_ACTIVE);
     }
     gatherProgress(entryDict, group, e, keys);
   }
@@ -1107,13 +1075,90 @@ SharedHandle<ValueBase> RemoveDownloadResultRpcMethod::process
 
   a2_gid_t gid = str2Gid(gidParam);
   if(!e->getRequestGroupMan()->removeDownloadResult(gid)) {
-    throw DL_ABORT_EX
-      (fmt("Could not remove download result of GID#%s",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("Could not remove download result of GID#%" PRId64 "", gid));
   }
   return VLB_OK;
 }
 
+namespace {
+void changeOption
+(const SharedHandle<RequestGroup>& group,
+ const Option& option,
+ DownloadEngine* e)
+{
+  const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
+  const SharedHandle<Option>& grOption = group->getOption();
+  grOption->merge(option);
+  if(option.defined(PREF_CHECKSUM)) {
+    const std::string& checksum = grOption->get(PREF_CHECKSUM);
+    std::pair<Scip, Scip> p;
+    util::divide(p, checksum.begin(), checksum.end(), '=');
+    std::string hashType(p.first.first, p.first.second);
+    util::lowercase(hashType);
+    dctx->setDigest(hashType, util::fromHex(p.second.first, p.second.second));
+  }
+  if(option.defined(PREF_SELECT_FILE)) {
+    SegList<int> sgl;
+    util::parseIntSegments(sgl, grOption->get(PREF_SELECT_FILE));
+    sgl.normalize();
+    dctx->setFileFilter(sgl);
+  }
+  if(option.defined(PREF_SPLIT)) {
+    group->setNumConcurrentCommand(grOption->getAsInt(PREF_SPLIT));
+  }
+  if(option.defined(PREF_MAX_CONNECTION_PER_SERVER)) {
+    int maxConn = grOption->getAsInt(PREF_MAX_CONNECTION_PER_SERVER);
+    const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
+    for(std::vector<SharedHandle<FileEntry> >::const_iterator i = files.begin(),
+          eoi = files.end(); i != eoi; ++i) {
+      (*i)->setMaxConnectionPerServer(maxConn);
+    }
+  }
+  if(option.defined(PREF_DIR) || option.defined(PREF_OUT)) {
+    if(dctx->getFileEntries().size() == 1
+#ifdef ENABLE_BITTORRENT
+       && !dctx->hasAttribute(bittorrent::BITTORRENT)
+#endif // ENABLE_BITTORRENT
+       ) {
+      dctx->getFirstFileEntry()->setPath
+        (grOption->blank(PREF_OUT) ? A2STR::NIL :
+         util::applyDir(grOption->get(PREF_DIR), grOption->get(PREF_OUT)));
+    }
+  }
+#ifdef ENABLE_BITTORRENT
+  if(option.defined(PREF_DIR) || option.defined(PREF_INDEX_OUT)) {
+    if(dctx->hasAttribute(bittorrent::BITTORRENT)) {
+      std::istringstream indexOutIn(grOption->get(PREF_INDEX_OUT));
+      std::vector<std::pair<size_t, std::string> > indexPaths =
+        util::createIndexPaths(indexOutIn);
+      for(std::vector<std::pair<size_t, std::string> >::const_iterator i =
+            indexPaths.begin(), eoi = indexPaths.end(); i != eoi; ++i) {
+        dctx->setFilePathWithIndex
+          ((*i).first,
+           util::applyDir(grOption->get(PREF_DIR), (*i).second));
+      }
+    }
+  }
+#endif // ENABLE_BITTORRENT
+  if(option.defined(PREF_MAX_DOWNLOAD_LIMIT)) {
+    group->setMaxDownloadSpeedLimit
+      (grOption->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
+  }
+  if(option.defined(PREF_MAX_UPLOAD_LIMIT)) {
+    group->setMaxUploadSpeedLimit(grOption->getAsInt(PREF_MAX_UPLOAD_LIMIT));
+  }
+#ifdef ENABLE_BITTORRENT
+  const SharedHandle<BtObject>& btObject =
+    e->getBtRegistry()->get(group->getGID());
+  if(btObject) {
+    if(option.defined(PREF_BT_MAX_PEERS)) {
+      btObject->btRuntime->setMaxPeers(grOption->getAsInt(PREF_BT_MAX_PEERS));
+    }
+  }
+#endif // ENABLE_BITTORRENT
+}
+} // namespace
+
 SharedHandle<ValueBase> ChangeOptionRpcMethod::process
 (const RpcRequest& req, DownloadEngine* e)
 {
@@ -1121,31 +1166,18 @@ SharedHandle<ValueBase> ChangeOptionRpcMethod::process
   const Dict* optsParam = checkRequiredParam<Dict>(req, 1);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
-  if(!group) {
-    throw DL_ABORT_EX
-      (fmt("Cannot change option for GID#%s",
-           util::itos(gid).c_str()));
-  }
-  SharedHandle<Option> option(new Option());
-  gatherChangeableOption(option, optsParam);
-  applyChangeableOption(group->getOption().get(), option.get());
-  if(option->defined(PREF_MAX_DOWNLOAD_LIMIT)) {
-    group->setMaxDownloadSpeedLimit
-      (option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
-  }
-  if(option->defined(PREF_MAX_UPLOAD_LIMIT)) {
-    group->setMaxUploadSpeedLimit(option->getAsInt(PREF_MAX_UPLOAD_LIMIT));
-  }
-#ifdef ENABLE_BITTORRENT
-  BtObject btObject = e->getBtRegistry()->get(group->getGID());
-  if(!btObject.isNull()) {
-    if(option->defined(PREF_BT_MAX_PEERS)) {
-      btObject.btRuntime_->setMaxPeers(option->getAsInt(PREF_BT_MAX_PEERS));
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
+  Option option;
+  if(group) {
+    if(group->getState() == RequestGroup::STATE_ACTIVE) {
+      gatherChangeableOption(&option, optsParam);
+    } else {
+      gatherChangeableOptionForReserved(&option, optsParam);
     }
+    changeOption(group, option, e);
+  } else {
+    throw DL_ABORT_EX(fmt("Cannot change option for GID#%" PRId64, gid));
   }
-#endif // ENABLE_BITTORRENT
   return VLB_OK;
 }
 
@@ -1154,28 +1186,32 @@ SharedHandle<ValueBase> ChangeGlobalOptionRpcMethod::process
 {
   const Dict* optsParam = checkRequiredParam<Dict>(req, 0);
 
-  SharedHandle<Option> option(new Option());
-  gatherChangeableGlobalOption(option, optsParam);
-  applyChangeableGlobalOption(e->getOption(), option.get());
+  Option option;
+  gatherChangeableGlobalOption(&option, optsParam);
+  e->getOption()->merge(option);
 
-  if(option->defined(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)) {
+  if(option.defined(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)) {
     e->getRequestGroupMan()->setMaxOverallDownloadSpeedLimit
-      (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT));
+      (option.getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT));
   }
-  if(option->defined(PREF_MAX_OVERALL_UPLOAD_LIMIT)) {
+  if(option.defined(PREF_MAX_OVERALL_UPLOAD_LIMIT)) {
     e->getRequestGroupMan()->setMaxOverallUploadSpeedLimit
-      (option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT));
+      (option.getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT));
   }
-  if(option->defined(PREF_MAX_CONCURRENT_DOWNLOADS)) {
+  if(option.defined(PREF_MAX_CONCURRENT_DOWNLOADS)) {
     e->getRequestGroupMan()->setMaxSimultaneousDownloads
-      (option->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS));
+      (option.getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS));
     e->getRequestGroupMan()->requestQueueCheck();
   }
-  if(option->defined(PREF_LOG_LEVEL)) {
-    LogFactory::setLogLevel(option->get(PREF_LOG_LEVEL));
+  if(option.defined(PREF_MAX_DOWNLOAD_RESULT)) {
+    e->getRequestGroupMan()->setMaxDownloadResult
+      (option.getAsInt(PREF_MAX_DOWNLOAD_RESULT));
   }
-  if(option->defined(PREF_LOG)) {
-    LogFactory::setLogFile(option->get(PREF_LOG));
+  if(option.defined(PREF_LOG_LEVEL)) {
+    LogFactory::setLogLevel(option.get(PREF_LOG_LEVEL));
+  }
+  if(option.defined(PREF_LOG)) {
+    LogFactory::setLogFile(option.get(PREF_LOG));
     try {
       LogFactory::reconfigure();
     } catch(RecoverableException& e) {
@@ -1204,15 +1240,16 @@ SharedHandle<ValueBase> GetVersionRpcMethod::process
 }
 
 namespace {
-template<typename InputIterator>
 void pushRequestOption
 (const SharedHandle<Dict>& dict,
- InputIterator optionFirst, InputIterator optionLast)
+ const SharedHandle<Option>& option,
+ const SharedHandle<OptionParser>& oparser)
 {
-  const std::set<std::string>& requestOptions = listRequestOptions();
-  for(; optionFirst != optionLast; ++optionFirst) {
-    if(requestOptions.count((*optionFirst).first)) {
-      dict->put((*optionFirst).first, (*optionFirst).second);
+  for(size_t i = 1, len = option::countOption(); i < len; ++i) {
+    const Pref* pref = option::i2p(i);
+    const SharedHandle<OptionHandler>& h = oparser->find(pref);
+    if(h && h->getInitialOption() && option->defined(pref)) {
+      dict->put(pref->k, option->get(pref));
     }
   }
 }
@@ -1224,16 +1261,13 @@ SharedHandle<ValueBase> GetOptionRpcMethod::process
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX
-      (fmt("Cannot get option for GID#%s",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("Cannot get option for GID#%" PRId64 "", gid));
   }
   SharedHandle<Dict> result = Dict::g();
   SharedHandle<Option> option = group->getOption();
-  pushRequestOption(result, option->begin(), option->end());
+  pushRequestOption(result, option, getOptionParser());
   return result;
 }
 
@@ -1241,11 +1275,14 @@ SharedHandle<ValueBase> GetGlobalOptionRpcMethod::process
 (const RpcRequest& req, DownloadEngine* e)
 {
   SharedHandle<Dict> result = Dict::g();
-  for(std::map<std::string, std::string>::const_iterator i =
-        e->getOption()->begin(), eoi = e->getOption()->end(); i != eoi; ++i) {
-    SharedHandle<OptionHandler> h = getOptionParser()->findByName((*i).first);
-    if(h && !h->isHidden()) {
-      result->put((*i).first, (*i).second);
+  for(size_t i = 0, len = e->getOption()->getTable().size(); i < len; ++i) {
+    const Pref* pref = option::i2p(i);
+    if(!e->getOption()->defined(pref)) {
+      continue;
+    }
+    const SharedHandle<OptionHandler>& h = getOptionParser()->find(pref);
+    if(h) {
+      result->put(pref->k, e->getOption()->get(pref));
     }
   }
   return result;
@@ -1291,11 +1328,9 @@ SharedHandle<ValueBase> GetServersRpcMethod::process
   const String* gidParam = checkRequiredParam<String>(req, 0);
 
   a2_gid_t gid = str2Gid(gidParam);
-  SharedHandle<RequestGroup> group =
-    e->getRequestGroupMan()->findRequestGroup(gid);
-  if(!group) {
-    throw DL_ABORT_EX(fmt("No active download for GID#%s",
-                          util::itos(gid).c_str()));
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
+  if(!group || group->getState() != RequestGroup::STATE_ACTIVE) {
+    throw DL_ABORT_EX(fmt("No active download for GID#%" PRId64, gid));
   }
   const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
   const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
@@ -1306,9 +1341,9 @@ SharedHandle<ValueBase> GetServersRpcMethod::process
     SharedHandle<Dict> fileEntry = Dict::g();
     fileEntry->put(KEY_INDEX, util::uitos(index));
     SharedHandle<List> servers = List::g();
-    const std::deque<SharedHandle<Request> >& requests =
+    const FileEntry::InFlightRequestSet& requests =
       (*fi)->getInFlightRequests();
-    for(std::deque<SharedHandle<Request> >::const_iterator ri =requests.begin(),
+    for(FileEntry::InFlightRequestSet::iterator ri =requests.begin(),
           eoi = requests.end(); ri != eoi; ++ri) {
       SharedHandle<PeerStat> ps = (*ri)->getPeerStat();
       if(ps) {
@@ -1316,7 +1351,7 @@ SharedHandle<ValueBase> GetServersRpcMethod::process
         serverEntry->put(KEY_URI, (*ri)->getUri());
         serverEntry->put(KEY_CURRENT_URI, (*ri)->getCurrentUri());
         serverEntry->put(KEY_DOWNLOAD_SPEED,
-                         util::uitos(ps->calculateDownloadSpeed()));
+                         util::itos(ps->calculateDownloadSpeed()));
         servers->append(serverEntry);
       }
     }
@@ -1330,7 +1365,7 @@ SharedHandle<ValueBase> ChangeUriRpcMethod::process
 (const RpcRequest& req, DownloadEngine* e)
 {
   const String* gidParam = checkRequiredParam<String>(req, 0);
-  const Integer* indexParam = checkRequiredParam<Integer>(req, 1);
+  const Integer* indexParam = checkRequiredInteger(req, 1, IntegerGE(1));
   const List* delUrisParam = checkRequiredParam<List>(req, 2);
   const List* addUrisParam = checkRequiredParam<List>(req, 3);
   const Integer* posParam = checkParam<Integer>(req, 4);
@@ -1339,12 +1374,9 @@ SharedHandle<ValueBase> ChangeUriRpcMethod::process
   bool posGiven = checkPosParam(posParam);
   size_t pos = posGiven ? posParam->i() : 0;
   size_t index = indexParam->i()-1;
-  SharedHandle<RequestGroup> group =
-    findRequestGroup(e->getRequestGroupMan(), gid);
+  SharedHandle<RequestGroup> group = e->getRequestGroupMan()->findGroup(gid);
   if(!group) {
-    throw DL_ABORT_EX
-      (fmt("Cannot remove URIs from GID#%s",
-           util::itos(gid).c_str()));
+    throw DL_ABORT_EX(fmt("Cannot remove URIs from GID#%" PRId64 "", gid));
   }
   const SharedHandle<DownloadContext>& dctx = group->getDownloadContext();
   const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
@@ -1421,8 +1453,8 @@ SharedHandle<ValueBase> GetGlobalStatRpcMethod::process
   const SharedHandle<RequestGroupMan>& rgman = e->getRequestGroupMan();
   TransferStat ts = rgman->calculateStat();
   SharedHandle<Dict> res = Dict::g();
-  res->put(KEY_DOWNLOAD_SPEED, util::uitos(ts.downloadSpeed));
-  res->put(KEY_UPLOAD_SPEED, util::uitos(ts.uploadSpeed));
+  res->put(KEY_DOWNLOAD_SPEED, util::itos(ts.downloadSpeed));
+  res->put(KEY_UPLOAD_SPEED, util::itos(ts.uploadSpeed));
   res->put(KEY_NUM_WAITING, util::uitos(rgman->getReservedGroups().size()));
   res->put(KEY_NUM_STOPPED, util::uitos(rgman->getDownloadResults().size()));
   res->put(KEY_NUM_ACTIVE, util::uitos(rgman->getRequestGroups().size()));
