@@ -32,28 +32,25 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_STREAM_CHECK_INTEGRITY_ENTRY_H_
-#define _D_STREAM_CHECK_INTEGRITY_ENTRY_H_
+#ifndef D_STREAM_CHECK_INTEGRITY_ENTRY_H
+#define D_STREAM_CHECK_INTEGRITY_ENTRY_H
 
 #include "PieceHashCheckIntegrityEntry.h"
-#include "TimeA2.h"
 
 namespace aria2 {
 
 class StreamCheckIntegrityEntry:public PieceHashCheckIntegrityEntry
 {
-private:
-  Time _timer;
 public:
   StreamCheckIntegrityEntry(RequestGroup* requestGroup,
                             Command* nextCommand = 0);
 
   virtual ~StreamCheckIntegrityEntry();
 
-  virtual void onDownloadFinished(std::deque<Command*>& commands,
+  virtual void onDownloadFinished(std::vector<Command*>& commands,
                                   DownloadEngine* e);
 
-  virtual void onDownloadIncomplete(std::deque<Command*>& commands,
+  virtual void onDownloadIncomplete(std::vector<Command*>& commands,
                                     DownloadEngine* e);
 };
 
@@ -61,4 +58,4 @@ typedef SharedHandle<StreamCheckIntegrityEntry> StreamCheckIntegrityEntryHandle;
 
 } // namespace aria2
 
-#endif // _D_STREAM_CHECK_INTEGRITY_ENTRY_H_
+#endif // D_STREAM_CHECK_INTEGRITY_ENTRY_H
