@@ -35,17 +35,6 @@
 #ifndef D_COMMON_H
 #define D_COMMON_H
 
-#ifdef __MINGW32__
-# undef SIZE_MAX
-# ifndef _OFF_T_
-#  define _OFF_T_
-typedef long long _off_t;
-#  ifndef _NO_OLDNAMES
-typedef _off_t off_t;
-#  endif // !_NO_OLDNAMES
-# endif // !_OFF_T_
-#endif // __MINGW32__
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -85,6 +74,18 @@ typedef _off_t off_t;
 // use C99 limit macros
 #define __STDC_LIMIT_MACROS
 // included here for compatibility issues with old compiler/libraries.
-#include <stdint.h>
+#ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#endif // HAVE_STDINT_H
+
+// For PRId64
+#define __STDC_FORMAT_MACROS
+#ifdef HAVE_INTTYPES_H
+#  include <inttypes.h>
+#endif // HAVE_INTTYPES_H
+
+#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
 
 #endif // D_COMMON_H
