@@ -148,11 +148,11 @@ void BittorrentHelperTest::testGetPieceHash() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
   load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
-  CPPUNIT_ASSERT_EQUAL(util::toHex("AAAAAAAAAAAAAAAAAAAA", 20),
+  CPPUNIT_ASSERT_EQUAL(std::string("AAAAAAAAAAAAAAAAAAAA"),
                        dctx->getPieceHash(0));
-  CPPUNIT_ASSERT_EQUAL(util::toHex("BBBBBBBBBBBBBBBBBBBB", 20),
+  CPPUNIT_ASSERT_EQUAL(std::string("BBBBBBBBBBBBBBBBBBBB"),
                        dctx->getPieceHash(1));
-  CPPUNIT_ASSERT_EQUAL(util::toHex("CCCCCCCCCCCCCCCCCCCC", 20),
+  CPPUNIT_ASSERT_EQUAL(std::string("CCCCCCCCCCCCCCCCCCCC"),
                        dctx->getPieceHash(2));
   CPPUNIT_ASSERT_EQUAL(std::string(""),
                        dctx->getPieceHash(3));
@@ -162,6 +162,7 @@ void BittorrentHelperTest::testGetPieceHash() {
 
 void BittorrentHelperTest::testGetFileEntries() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
+  option_->put(PREF_MAX_CONNECTION_PER_SERVER, "10");
   load(A2_TEST_DIR"/test.torrent", dctx, option_);
   // This is multi-file torrent.
   std::vector<SharedHandle<FileEntry> > fileEntries = dctx->getFileEntries();
@@ -174,14 +175,17 @@ void BittorrentHelperTest::testGetFileEntries() {
                        fileEntry1->getPath());
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-test/aria2/src/aria2c"),
                        fileEntry1->getOriginalName());
+  CPPUNIT_ASSERT_EQUAL(10, fileEntry1->getMaxConnectionPerServer());
   itr++;
   SharedHandle<FileEntry> fileEntry2 = *itr;
   CPPUNIT_ASSERT_EQUAL(std::string("./aria2-test/aria2-0.2.2.tar.bz2"),
                        fileEntry2->getPath());
+  CPPUNIT_ASSERT_EQUAL(10, fileEntry2->getMaxConnectionPerServer());
 }
 
 void BittorrentHelperTest::testGetFileEntriesSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
+  option_->put(PREF_MAX_CONNECTION_PER_SERVER, "10");
   load(A2_TEST_DIR"/single.torrent", dctx, option_);
   // This is multi-file torrent.
   std::vector<SharedHandle<FileEntry> > fileEntries = dctx->getFileEntries();
@@ -194,20 +198,21 @@ void BittorrentHelperTest::testGetFileEntriesSingle() {
                        fileEntry1->getPath());
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-0.8.2.tar.bz2"),
                        fileEntry1->getOriginalName());
+  CPPUNIT_ASSERT_EQUAL(10, fileEntry1->getMaxConnectionPerServer());
 }
 
 void BittorrentHelperTest::testGetTotalLength() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
   load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, dctx->getTotalLength());
+  CPPUNIT_ASSERT_EQUAL((int64_t)384LL, dctx->getTotalLength());
 }
 
 void BittorrentHelperTest::testGetTotalLengthSingle() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
   load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)384ULL, dctx->getTotalLength());
+  CPPUNIT_ASSERT_EQUAL((int64_t)384LL, dctx->getTotalLength());
 }
 
 void BittorrentHelperTest::testGetFileModeMulti() {
@@ -288,7 +293,7 @@ void BittorrentHelperTest::testGetPieceLength() {
   SharedHandle<DownloadContext> dctx(new DownloadContext());
   load(A2_TEST_DIR"/test.torrent", dctx, option_);
 
-  CPPUNIT_ASSERT_EQUAL((size_t)128, dctx->getPieceLength());
+  CPPUNIT_ASSERT_EQUAL(128, dctx->getPieceLength());
 }
 
 void BittorrentHelperTest::testGetInfoHashAsString() {
@@ -412,7 +417,7 @@ void BittorrentHelperTest::testLoadFromMemory_multiFileNonUtf8Path()
 {
   SharedHandle<List> path = List::g();
   path->append("path");
-  path->append(util::fromHex("90a28a")+"E");
+  path->append(fromHex("90a28a")+"E");
   SharedHandle<Dict> file = Dict::g();
   file->put("length", Integer::g(1024));
   file->put("path", path);
@@ -422,7 +427,7 @@ void BittorrentHelperTest::testLoadFromMemory_multiFileNonUtf8Path()
   info->put("files", files);
   info->put("piece length", Integer::g(1024));
   info->put("pieces", "01234567890123456789");
-  info->put("name", util::fromHex("1b")+"$B%O%m!<"+util::fromHex("1b")+"(B");
+  info->put("name", fromHex("1b")+"$B%O%m!<"+fromHex("1b")+"(B");
   Dict dict;
   dict.put("info", info);
   SharedHandle<DownloadContext> dctx(new DownloadContext());
@@ -441,7 +446,7 @@ void BittorrentHelperTest::testLoadFromMemory_singleFileNonUtf8Path()
   SharedHandle<Dict> info = Dict::g();
   info->put("piece length", Integer::g(1024));
   info->put("pieces", "01234567890123456789");
-  info->put("name", util::fromHex("90a28a")+"E");
+  info->put("name", fromHex("90a28a")+"E");
   info->put("length", Integer::g(1024));
   Dict dict;
   dict.put("info", info);
@@ -645,16 +650,20 @@ void BittorrentHelperTest::testSetFileFilter_single()
   load(A2_TEST_DIR"/single.torrent", dctx, option_);
 
   CPPUNIT_ASSERT(dctx->getFirstFileEntry()->isRequested());
-
-  dctx->setFileFilter(util::parseIntRange(""));
+  SegList<int> sgl;
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(dctx->getFirstFileEntry()->isRequested());
 
-  dctx->setFileFilter(util::parseIntRange("1"));
+  sgl.clear();
+  sgl.add(1, 2);
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(dctx->getFirstFileEntry()->isRequested());
 
   // For single file torrent, file is always selected whatever range
   // is passed.
-  dctx->setFileFilter(util::parseIntRange("2"));
+  sgl.clear();
+  sgl.add(2, 3);
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(dctx->getFirstFileEntry()->isRequested());
 }
 
@@ -666,19 +675,25 @@ void BittorrentHelperTest::testSetFileFilter_multi()
   CPPUNIT_ASSERT(dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(dctx->getFileEntries()[1]->isRequested());
 
-  dctx->setFileFilter(util::parseIntRange(""));
+  SegList<int> sgl;
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(dctx->getFileEntries()[1]->isRequested());
 
-  dctx->setFileFilter(util::parseIntRange("2"));
+  sgl.add(2, 3);
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(!dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(dctx->getFileEntries()[1]->isRequested());
 
-  dctx->setFileFilter(util::parseIntRange("3"));
+  sgl.clear();
+  sgl.add(3, 4);
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(!dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(!dctx->getFileEntries()[1]->isRequested());
 
-  dctx->setFileFilter(util::parseIntRange("1,2"));
+  sgl.clear();
+  util::parseIntSegments(sgl, "1,2");
+  dctx->setFileFilter(sgl);
   CPPUNIT_ASSERT(dctx->getFileEntries()[0]->isRequested());
   CPPUNIT_ASSERT(dctx->getFileEntries()[1]->isRequested());
 }
@@ -771,7 +786,7 @@ void BittorrentHelperTest::testParseMagnet()
 void BittorrentHelperTest::testParseMagnet_base32()
 {
   std::string infoHash = "248d0a1cd08284299de78d5c1ed359bb46717d8c";
-  std::string base32InfoHash = base32::encode(util::fromHex(infoHash));
+  std::string base32InfoHash = base32::encode(fromHex(infoHash));
   std::string magnet = "magnet:?xt=urn:btih:"+base32InfoHash+"&dn=aria2";
   SharedHandle<TorrentAttribute> attrs = bittorrent::parseMagnet(magnet);
   CPPUNIT_ASSERT_EQUAL
@@ -814,7 +829,7 @@ void BittorrentHelperTest::testExtractPeerFromString()
 {
   std::string hextext = "100210354527354678541237324732171ae1";
   hextext += "20010db8bd0501d2288a1fc0000110ee1ae2";
-  std::string peersstr = "36:"+util::fromHex(hextext);
+  std::string peersstr = "36:"+fromHex(hextext);
   SharedHandle<ValueBase> str = bencode2::decode(peersstr);
   std::deque<SharedHandle<Peer> > peers;
   extractPeer(str, AF_INET6, std::back_inserter(peers));
@@ -828,7 +843,7 @@ void BittorrentHelperTest::testExtractPeerFromString()
 
   hextext = "c0a800011ae1";
   hextext += "c0a800021ae2";
-  peersstr = "12:"+util::fromHex(hextext);
+  peersstr = "12:"+fromHex(hextext);
   str = bencode2::decode(peersstr);
   peers.clear();
   extractPeer(str, AF_INET, std::back_inserter(peers));
@@ -836,7 +851,7 @@ void BittorrentHelperTest::testExtractPeerFromString()
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), peers[0]->getIPAddress());
   CPPUNIT_ASSERT_EQUAL((uint16_t)6881, peers[0]->getPort());
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.2"), peers[1]->getIPAddress());
-  CPPUNIT_ASSERT_EQUAL((uint16_t)6882, peers[1]->getPort()); 
+  CPPUNIT_ASSERT_EQUAL((uint16_t)6882, peers[1]->getPort());
 }
 
 void BittorrentHelperTest::testExtractPeerFromList()
@@ -846,7 +861,7 @@ void BittorrentHelperTest::testExtractPeerFromList()
     "4:porti2006eeee";
 
   SharedHandle<ValueBase> dict = bencode2::decode(peersString);
-  
+
   std::deque<SharedHandle<Peer> > peers;
   extractPeer(downcast<Dict>(dict)->get("peers"), AF_INET, std::back_inserter(peers));
   CPPUNIT_ASSERT_EQUAL((size_t)1, peers.size());
